/*	
	Name:		SNPsim (snpsimx.x.c)
	Purpose:	To simulate SNPs under a variety of relevant scenarios
	Code:		David Posada (DP) (with much help from Carsten Wiuf)		
	Started:	27 March 02 

	Version 0.1
		- Simulates genealogies under the coalescent with recombination
		- Assumes SNPs are biallelic (0,1)
		- Mutation model is JC (Jukes-Cantor) or ISM (Infinite Sites Model)
		- Simulates SNPs under theta or generates a fixed number
		- Simulate homogeneous recombination or recombination hotspots
		- Allow for several demographics
			- Exponential growth
			- Several periods
	
	Version 0.2 (8 October 2002)
		- Read everything as float although use some of them as integers
			(this way we can express -e1e4, for example)
		- Expected and fixed number of hotspots as decimals
		- Added heterogeneity of hotspot recombination rate (with gamma distribution)
		- Added Uniform hotspot (until now there were always Normal)
		- Added hotspot interference
	
	Version 0.3 (18 October 2002)
		- Bug in the hotspot interference implementation
		- Cleaned some code that I was not using anymore

	Version 0.4 (11 November 2002)
		- Version released to the web		
		- Counting only recombination events that happen in ancestral material
			that did not found its MRCA (X material). Before we counted X|X, X|0, 
			and 0|X. No we only count X|X
		- New option for not simulating data (just do coalescent)
		- Redefine the interference interval instead of m-1 as m (so now instead of -z3 we use -z4)
		
	Version 0.5 (30 January 2003)
		- Small bug fixed reading options -n and -x on command line (thanks to Clara Singer)
		- Change Ne, mu and noisy defaults (now -e10e4 -u1e-8 -y1)
		- By default print SNPs genotypes and haplotypes
		- Print memory available
		
	Version 0.6 (7 April 2003)
		- Inefficiency in the implementation of the ISM. Update the totalBranchSum after evolving each site. 
			(thanks to Jack Liu). Not clear that this was a bug favoring sites with high indexes.
		- Potential exception (array out ob bonds: hotspot[poissonNumber-1]) fixed (thanks to Jack Liu)
		- Debugging: When checking the distribution of mutations (CHECK_MUT_DISTRIBUTION functions) 
			calculate only with datasets with SNPs
			
	Version 0.7 (20 October 2003)
		- Made sure that all parameters are well parsed: read them as doubles or integers 
			or cast them afterwards
		- Change seed argument symbol from "*" to "#". Now the seed arguemnt should be passed as in -#34742, for example
		- Fixed: It would not accept demographics from command line

	Version 0.8 (6 May 2004)
		- Added PrintDefaults() function
		- Set seed to time(null) at the very beginning, just for having it in the defaults.
		- Comment main variables in the defaults section in the code
		- Count reps without SNPs		

	Credits:
		- Carsten Wiuf came up wit most of the theory and suggested many, many ideas
		- The first part of this program was done at BYU (Utah, US). The second part (and biggest)
			was done while working for Variagenics, Inc. (Mass, US). The last part of the 
			program was done at the University of Vigo (Spain)
		- Demographics implementation is based on RR Hudson's code
		- For reporting bugs: Clara Singer, Jack Liu, Kui Zhang
		
	To do:
		- Logistic growth
		- Island model
		- Keep trees for segments, not sites.
				

	GENERAL PUBLIC LICENSE	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>

#ifndef macintosh
	#include <sys/types.h>
	#include <sys/stat.h>
#endif

#ifdef macintosh
	#include <sioux.h>
	#include <console.h>
	#include <unix.h>
#endif

#define PROGRAM_NAME		"SNPsim"
#define VERSION_NUMBER		"version 0.8"
#define	MAX_NAME			30
#define	NO					0
#define	YES					1
#define	pos(i,j,n)			((i)*(n)+(j))

#define CHECK_MUT_DISTRIBUTION
#undef	CHECK_MUT_DISTRIBUTION


typedef struct node
	{
	struct node		*left, *right, *anc;
	int				index, isAncestralSite, label;
	double			length, time;
	}
	TreeNode;

typedef struct individual
	{	
	int haplotype1;
	int haplotype2;
	}
	IndividualStr;

typedef struct site
	{
	int		isJC;
	int		isSNP;
	int		done;
	double	branchSum;
	}
	SiteStr;


/* Prototypes */

static void 	PrintHeader (FILE *file);
static void 	PrintDate (FILE *file);
static void 	PrintUsage ();
static void 	PrintDefaults (FILE *fp);
static void		ReadUntil (FILE *fv, char stopChar, char *what);
static void		PrintRunSettings (FILE *filep, long int seed);
static void		ReadParametersFromFile ();
static void		ReadParametersFromCommandLine (int argc, char **argv);
static void		PrepareFiles ();
static void		PrintTrees ();
static void		WriteTree (TreeNode *p);
static void		PrintTimes ();
static void		ListTimes (int position);
static void		PrintMutationModel (int replicate);
static void		PrintSNPs ();
static void		PrintHaplotypes ();
static void 	PrintGenotypes ();
static void 	PrintCommandLine (FILE *fp, long int seed);
static void		MakeGenotypes (long int *seed);
static void		MakeCoalescenceTreeHotspotRec (int numGametes, int numSites, int N, long int *seed);
static void		MakeCoalescenceTreeHomoRec (int numGametes, int numSites, int N, long int *seed);
static int		Index (TreeNode *p);
static int 		Label (TreeNode *p);
static void 	EvolveHaplotypesOnTree (long int *seed, double m, int numSites);
static void		RelabelNodes (TreeNode *p);
static void		SimulateJCForSite (TreeNode *p, int siteNum, int numSites, double mutationRate, long int *seed);
static void 	SimulateISMForSite (TreeNode *p, int siteNum, int numSites, double m, long int *seed);
static void 	SimulateISM  (long int *seed, int numMutations);
static double	SumBranches (TreeNode *p);
static int 		CalcIndividualG (int who, int *giBegin, int *giEnd, int numSites, TreeNode *nodes, int *activeGametes);
static int 		IsValidBreakSite (int *activeGametes, int whichGamete, int whichSite, TreeNode *p);
static int	 	CountsForExpNumRec (int *activeGametes, int whichGamete, int whichSite, TreeNode *nodes, int *MRCA);
static int 		GammaHotspots (double lambda, double interval, long int *seed);
static int 		PoissonHotspots (double lambda, long int *seed);
static void		UniformHotspots (int numberHotspots, long int *seed);
static void 	SimRecombinationPDF (double *counts, long int *seed);
static int 		CountSNPs ();
static void 	PrintSequences ();
static double 	RandomUniform (long int *seed);
static int 		RandomUniformTo (int max, long int *seed);
static int		RandomPoisson (double lambda, long int *seed);
static double 	RandomNormal (double mean, double variance, long int *seed);
static double	RandomExponential (double mean, long int *seed);
static double	RandomGamma (double shape, long int *seed);
static double	RandomGamma1 (double s, long int *seed);
static double	RandomGamma2 (double s, long int *seed);
static void 	RandomizeArray (int array[],int first, int last, long int *seed);
static void		Sioux ();
static int		TryToAlloc (long request);
static long		MemAvailable (int atom); 
static void 	PrintMemory (FILE *filep);

/* Global variables */
TreeNode		*nodes, **treeRoot;
IndividualStr	*individual;
SiteStr			*site;
long int		userSeed;
static int		*matrix, *MRCA, *hotspot;
static int		tipLabel, intLabel;
static int		numGametes, N, *Nbegin, *Nend,  *cumDuration, numSites, numIndividuals, numDataSets, numPointsRecPDF, numPeriods;
static int		noisy, numNodes, numSitesOutside, numJCsites, numISMsites;
static int		numRE, numREblock, recNotToCount, numHotspots, numCA, numMU, numSNPs, zeroRec, zeroSNPs;
static double	expNumPoissonHotspots, expNumHotspots, fixedNumHotspots;
static double	meanNumSNPs, meanNumHotspots, meanNumHotspots, meanNumRE, meanC01, meanNumREblock, meanNumREtc, meanNumCA, meanNumMU;
static double 	cumNumRE, cumC01, cumNumREtc, cumNumREblock, cumNumHotspots, cumNumREntc, cumNumCA, cumNumMU, cumNumSNPs;
static double	cumNumRESq, cumC01Sq, cumNumREtcSq, cumNumREblockSq, cumNumHotspotsSq, cumNumMUSq, cumNumSNPsSq;
static double	varNumRE, varNumREtc, varC01, varNumREblock, varNumHotspots, varNumMU, varNumSNPs;
static double	expNumRE, expNumMU, expVarNumMU;
static double	theta, rho, tau, C01, alpha;
static double	mutationRate, JCrelMutationRate, recombinationRate, propJCsites, blockRecombinationRate, globalRecombinationRate, hotspotImprecision;
static char		SNPfile[MAX_NAME], haplotypeFile[MAX_NAME], sequenceFile[MAX_NAME], genotypeFile[MAX_NAME];
static char		treeFile[MAX_NAME], timesFile[MAX_NAME], mutationsFile[MAX_NAME], dirname[MAX_NAME];
static int		doPrintSequences, doPrintGenotypes, doPrintSNPs, doPrintHaplotypes, doPrintTrees, doPrintTimes, doPrintMutationModel, doBlockRecombination;
static int		doExponential, doLogistic, doDemographics, carryingCapacity, simData, simulateJustSNPs;
static int		hotspotHeterogeneity, hotspotInterference, interferenceInterval, hotspotUniform, hotspotWidth;
static double	*periodGrowth, growthRate;
static double	TMRCA, cumTMRCA, cumTMRCASq, meanTMRCA, expTMRCA, varTMRCA, expVarTMRCA;

#ifdef CHECK_MUT_DISTRIBUTION
	static int		*MutCount, *SiteMut, dataSetsWithSNPs;
	static double	SumPos, meanSumPos;
	FILE			*fpSites;
#endif

/* File pointers */
FILE			*fpSNPs, *fpHaplotypes, *fpGenotypes, *fpSequences, *fpTrees, *fpTimes, *fpMutations;

#ifdef macintosh
	main (void)
#else
	main (int argc, char **argv)
#endif
{
	FILE 		*fp;
	int			dataSetNum, i;
	#ifdef CHECK_MUT_DISTRIBUTION
		int		j;
	#endif
	long int	seed, originalSeed;
	double		a, a2;
	float 		start, secs;
	
 	/* Default settings */
 	numDataSets = 10;			/* the number of samples to simulate */
	numGametes = 8;				/* number of gametes in each data set */
	numIndividuals = 4;			/* number of individuals */
	numSites = 200;				/* number of sites (markers, loci) to simulate = length of the chromosomes */
	N = 1000;					/* effective population size */
	numPeriods = 0;				/* number of distinct demographic periods */
	doDemographics = NO;		/* whether to implement demographics */
	doExponential = NO;			/* wheter to do exponential growth */
	doLogistic = NO;			/* wheter to do logist growth (NOT implemented yet) */	
	growthRate = 0;				/* rate for the exponential growth */
	carryingCapacity = 0;		/* carrying capacity for logistic growth model*/
	recombinationRate = 0;  	/* background homogeneous recombination rate per site per generation*/
	blockRecombinationRate = 0; /* expected recombination rate at the hotspots sites */
	expNumPoissonHotspots = 0;  /* expected number of hotspots for a given sample in the absence of interference */
	expNumHotspots = 0;			/* expected number of hotspots considering interference */
	fixedNumHotspots = 0;		/* fixed number of hotspots for all samples */
	hotspotImprecision = 0;		/* variance of a Normal distribution around the hotspot center */
	hotspotWidth = 1;			/* the width of the hotspots when they are uniform */
	interferenceInterval = 1;	/* interference between hotspots [0-1:cluster, 1:no interference, >1: pushed away] */
	mutationRate = 1.0e-7;		/* nucleotide mutation rate per site per generation */
	propJCsites = 0;			/* proportion of sites that will mutate according to the JC model */
	JCrelMutationRate = 1.0;	/* relative rate JC/ISM for sites */
	numPointsRecPDF = 10000;	/* number of points for the probability density function for recombination*/
	hotspotHeterogeneity = NO;	/* whether there is hotspot heterogeneity */
	hotspotInterference = NO;	/* whether there is hotspot interference */
	hotspotUniform = NO;		/* whether hotspots are uniform */
	alpha = 0.5;				/* shape of the gamma distribution for hotspot recombination heterogeneity */
	noisy = 1;					/* level of information to be printed in the screen (see below) */
	numNodes = 1000;			/* number of nodes allocated to build the coalescent trees */
	doBlockRecombination = NO;	/* whether we will do heterogeneous recombination */
	simulateJustSNPs = NO;		/* whether to simualte just a fixed number of SNP sites*/
	doPrintSNPs = YES;			/* whether to print SNPs */
	doPrintHaplotypes = YES;	/* whether to print haplotypes */
 	doPrintSequences = NO;		/* whether to print sequences */
	doPrintGenotypes = NO;		/* whether to print genotypes */
	doPrintTrees = NO;			/* whether to print trees for each site */
	doPrintTimes = NO;			/* whether to print coalescent times */
 	doPrintMutationModel = NO;	/* whether to print info about the mutation model for each site */
	simData = YES;				/* whether to simulate any data (or just look at the expectations; useful for debugging) */
	seed = time(NULL); 			/* seed for the random number */
	userSeed = 0;				/* seed entered by the user*/
	
	/* Noisy level
	0: nothing
	1: run information summarizing the simulations
	2: run settings for each sample 
	3: calculation status and event (recombinations and coalescences) information
	4: ancestral status for each chromosome at each event + MRCA status
	5: potential recombining locations (g and G vectors)
	*/

	/* arrange file names */
	strcpy(SNPfile, "SNPs");
	strcpy(haplotypeFile, "haplotypes");
 	strcpy(genotypeFile, "genotypes");
 	strcpy(sequenceFile, "sequences");
 	strcpy(treeFile, "trees");
 	strcpy(timesFile, "times");
 	strcpy(mutationsFile, "mutations");
	
	
	/* in macintosh the arguments are read from a file. For the other OS
	arguments are given at the command line */
	#ifdef macintosh
		_fcreator = 'R*ch';
		_ftype = 'TEXT';
		if ((fp = freopen("parameters", "r", stdin)) == NULL) 
			{
			fprintf (stderr, "\nERROR: Can't read parameters file.");
			PrintUsage();
			exit(-1);
			}
		ReadParametersFromFile();
		Sioux();
		fclose(fp);
	#else
	ReadParametersFromCommandLine (argc, argv);
		if (argc < 2)
			{
			if ((fp = freopen("parameters", "r", stdin)) != NULL) 
				ReadParametersFromFile();
			else
				{
				fprintf (stderr, "\nERROR: No parameters specified (use command line or parameter file)");
				PrintUsage();
				exit(-1);
				}
			}
	#endif

	start = clock();
	if (noisy > 0)
		{
		PrintHeader(stderr);
 		PrintDate(stderr);
		PrintMemory (stderr);	
		}
		
	if (simData == NO)
		{
		doPrintSNPs = NO;
	 	doPrintHaplotypes = NO;
	 	doPrintSequences = NO;
		doPrintGenotypes = NO;
		doPrintTrees = NO;
		doPrintTimes = NO;
	 	doPrintMutationModel = NO;
		}

	if (simData == YES)
		PrepareFiles();
		
	/* initialize some variables */
	a = 0;
	a2 = 0;
	for (i=1; i<numGametes; i++)
		{
		a += 1.0/(double)i; 
		a2 += 1.0/(double)pow(i,2); 
		}
	/* rho is the homogeneous recombination rate per gene per 4N generations */	
	rho = 4.0 * N * recombinationRate * numSites;	
	expNumHotspots = expNumPoissonHotspots / (double) (interferenceInterval);
	if (fixedNumHotspots == 0)
		tau = 4.0 * N * blockRecombinationRate * expNumHotspots; /* corresponds to the block recombination */
	else
		tau = 4.0 * N * blockRecombinationRate * fixedNumHotspots; /* corresponds to the block recombination */
	
	if (hotspotUniform == NO)
		numSitesOutside = 10 * sqrt (hotspotImprecision);
	else
		numSitesOutside =  hotspotWidth / 2.0;
	
	expNumRE = (rho + tau) * a;

/*	theta = 4.0 * N * mutationRate * numSites;*/
	theta = 4.0 * N * mutationRate * numSites * ((1.0-propJCsites) + JCrelMutationRate*propJCsites);

	expNumMU = theta * a;
	expVarNumMU =  theta * a + pow(theta,2) * a2;
	
	expTMRCA = 2 * (1 - 1.0/numGametes) /* 2.0 * N*/;
	expVarTMRCA = 0;
	for (i=2; i<= numGametes; i++)
		expVarTMRCA += /*4.0 * pow(N,2) */ 4.0 / (pow(i,2) * pow(i-1,2));

	if (fixedNumHotspots == 0)
		globalRecombinationRate = recombinationRate + blockRecombinationRate*expNumHotspots/(double)numSites;
	else
		globalRecombinationRate = recombinationRate + blockRecombinationRate*fixedNumHotspots/(double)numSites;

  	/* Set seed and spin wheels of pseudorandom number generator */
	/* seed = (unsigned int) clock();*/
	if (userSeed > 0)
		seed = userSeed;
	
	originalSeed = seed;
	
	for (i=0; i<100; i++)	
		RandomUniform(&seed);

	cumNumRE = cumNumRESq = cumNumREtc = cumNumREtcSq = cumC01 = cumC01Sq = cumNumREblock = cumNumREblockSq	= 0;
	cumNumCA = cumNumMU	= cumNumMUSq = cumNumREntc = cumNumSNPs	= cumNumSNPsSq= cumNumHotspots = 0;
	cumNumHotspotsSq = zeroRec = zeroSNPs= cumTMRCA = cumTMRCASq = 0;

	if (doPrintSNPs == YES)
		{
		fprintf (fpSNPs, "%s - ",PROGRAM_NAME);
		PrintDate (fpSNPs);
		PrintCommandLine (fpSNPs, originalSeed);
		fprintf (fpSNPs,"\n%d", numDataSets);
		fprintf (fpSNPs,"\n%d %d\n", numIndividuals, numSites);

		fprintf (fpHaplotypes, "%s - ",PROGRAM_NAME);
		PrintDate (fpHaplotypes);
		PrintCommandLine (fpHaplotypes, originalSeed);
		fprintf (fpHaplotypes,"\n%d", numDataSets);
		fprintf (fpHaplotypes,"\n%d %d\n", numIndividuals, numSites);
		}

  	/* Check the distribution of the mutations */
  	#ifdef CHECK_MUT_DISTRIBUTION
  		MutCount = (int *) calloc(numSites, sizeof(int));
		dataSetsWithSNPs = 0;
		meanSumPos = 0;
 		fpSites = fopen ("mutatedSites","w");
 	#endif

	for (dataSetNum=0; dataSetNum<numDataSets; dataSetNum++)
		{
		if (noisy == 1)
			{
			fprintf (stdout, "\rReplicate #%3d/%d", dataSetNum+1, numDataSets);
			fflush (stdout);
			}
		matrix = (int*) calloc (2*(numGametes+1)*numSites, sizeof(int));
		if (!matrix)
			{
			fprintf (stderr, "Could not allocate matrix\n");
			exit (1);
			}	
		site = (SiteStr*) calloc (numSites, sizeof(SiteStr));
		if (!site)
			{
			fprintf (stderr, "Could not allocate site\n");
			exit (1);
			}	
		numRE = C01 = numREblock= recNotToCount = numCA = numMU = TMRCA = 0;
	
		if (simulateJustSNPs == NO)
			numSNPs = 0;
	
		if (noisy > 2)
			fprintf (stderr, "\n>> Start making coalescent tree... \n");

		if (doBlockRecombination == YES)
			MakeCoalescenceTreeHotspotRec (numGametes, numSites, N, &seed);
		else
			MakeCoalescenceTreeHomoRec (numGametes, numSites, N, &seed);
				
		if (numRE == 0)
			zeroRec++;
		cumNumRE += numRE;
		cumNumRESq += pow(numRE,2);
		cumC01 += C01;
		cumC01Sq += pow(C01,2);
		cumNumREtc += (numRE - recNotToCount);
		cumNumREtcSq += pow((numRE - recNotToCount),2);
		cumNumREblock += numREblock;
		cumNumREblockSq += pow(numREblock,2);
		cumNumREntc += recNotToCount;
		cumNumCA += numCA;
		cumNumHotspots += numHotspots;
		cumNumHotspotsSq += pow(numHotspots,2);
		cumTMRCA += TMRCA;
		cumTMRCASq += pow(TMRCA,2);

		if (simData == YES)
			{
			if (noisy > 2)
				fprintf (stderr, "\n>> Start simulating sequences... \n");	
			
			EvolveHaplotypesOnTree (&seed, mutationRate, numSites);
			if (simulateJustSNPs == NO)
				numSNPs = CountSNPs();
			
			cumNumMU += numMU;
			cumNumMUSq += pow(numMU,2);
			cumNumSNPs += numSNPs;
			cumNumSNPsSq += pow(numSNPs,2);
			}
			
		if (noisy > 1)
			{
			fprintf (stderr, "\nData set %d",dataSetNum+1);
			fprintf (stderr, "\nNumber of total recombination events =   %d (%d countable)", numRE, numRE - recNotToCount);
			if (doBlockRecombination == YES)
				{
				fprintf (stderr,"\n%d hotspots = ", numHotspots);
				for (i=0; i<numHotspots; i++)
					fprintf (stderr, "%d ", hotspot[i]+1);
				}
			fprintf (stderr, "\nNumber of coalescence events   =   %d", numCA);
			fprintf (stderr, "\nNumber of mutational events    =   %d", numMU);
			fprintf (stderr, "\nNumber of SNPs                 =   %d\n\n", numSNPs);
			}

		if (simData == YES)
			{
			MakeGenotypes (&seed);

			if (numSNPs == 0)
				zeroSNPs++;
			
			if (doPrintSNPs == YES && numSNPs > 0) /* we only print replicates with variation */
				{
				fprintf (fpSNPs, "#%d\n", dataSetNum+1);
				PrintSNPs ();
				}
			if (doPrintHaplotypes == YES && numSNPs > 0) /* we only print replicates with variation */
				{
				fprintf (fpHaplotypes, "#%d\n", dataSetNum+1);
				PrintHaplotypes ();
				}
			if (doPrintSequences == YES)
				{
				fprintf (fpSequences, "#%d\n", dataSetNum+1);
				PrintSequences (dataSetNum);
				}
			if (doPrintGenotypes == YES )
				{
				fprintf (fpGenotypes, "#%d\n", dataSetNum+1);
				PrintGenotypes ();
				}
			if (doPrintMutationModel == YES)
				PrintMutationModel (dataSetNum);	
			if (doPrintTrees == YES)
				PrintTrees (dataSetNum);			
			if (doPrintTimes == YES)
				PrintTimes (dataSetNum);
			
			 #ifdef CHECK_MUT_DISTRIBUTION
			/* Check expectations regarding the position of the mutations in the tree */
			/*	Find out how many mutations are per site 
				we expect theta/i number of sites with i mutations
				count how many sites have 1, 2, 3..10 mutations */

			SiteMut = (int *) calloc(numSites, sizeof(int));
			for (i=0; i<numGametes; i++)
				for (j=0; j<numSites; j++)
					if (matrix[pos(i,j,numSites)] == 1)
						SiteMut[j]++;
			for (i=1; i<=10; i++)
				for (j=0; j<numSites; j++)
					if (SiteMut[j] == i)
						MutCount[i]++;

			/* Check expectations regarding the distribution of the mutations along the sequences */
			/* the average sum of the positions of the segregating sites has to be L/2 */
			SumPos = 0;
			if (numSNPs > 0)
				{
				dataSetsWithSNPs++;
				for (j=0; j<numSites; j++)
					if (site[j].isSNP == YES)
						SumPos += (j+1);
					meanSumPos += (SumPos / (double) numSNPs);
				}
			#endif
			}

		free (nodes);
		free (treeRoot);
		free (matrix);
		free (individual);			
		
		/* if(((dataSetNum*10)%numDataSets) == 0)
			PrintMemory(stderr); */
		}

	meanNumRE  = cumNumRE /  numDataSets;
	meanNumREtc  = (cumNumRE - cumNumREntc) /  numDataSets;
	meanC01  = cumC01 /  numDataSets;
	meanNumREblock  = cumNumREblock /  numDataSets;
	meanNumCA  = cumNumCA /  numDataSets;
	meanNumMU  = cumNumMU /  numDataSets;
	meanNumSNPs  = cumNumSNPs /  numDataSets;
	meanNumHotspots = cumNumHotspots / numDataSets;
	meanTMRCA  =  cumTMRCA /  numDataSets;

	varNumRE = (1.0 / (double) (numDataSets-1)) * (cumNumRESq - pow(cumNumRE,2) / (double) numDataSets);
	varNumREtc = (1.0 / (double) (numDataSets-1)) * (cumNumREtcSq - pow(cumNumREtc,2) / (double) numDataSets);
	varC01 = (1.0 / (double) (numDataSets-1)) * (cumC01Sq - pow(cumC01,2) / (double) numDataSets);
	varNumREblock = (1.0 / (double) (numDataSets-1)) * (cumNumREblockSq - pow(cumNumREblock,2) / (double) numDataSets);
	varNumHotspots = (1.0 / (double) (numDataSets-1)) * (cumNumHotspotsSq - pow(cumNumHotspots,2) / (double) numDataSets);
	varNumMU = (1.0 / (double) (numDataSets-1)) * (cumNumMUSq - pow(cumNumMU,2) / (double) numDataSets);
	varNumSNPs = (1.0 / (double) (numDataSets-1)) * (cumNumSNPsSq - pow(cumNumSNPs,2) / (double) numDataSets);
	varTMRCA = (1.0 / (double) (numDataSets-1)) * (cumTMRCASq - pow(cumTMRCA,2) / (double) numDataSets);
	
	if (noisy > 0)
		{
		/*if (noisy < 2)
			PrintHeader (stdout);*/

		fprintf (stdout, "\n\n** Simulations finished at ");
		PrintDate (stdout);
		PrintRunSettings (stdout, originalSeed);

		if (simData == YES)
			{
			fprintf (stdout, "\n\n\nOutput files are in folder \"%s\":", dirname);
			if (doPrintSNPs == YES)
				{
				fprintf (stdout, "\n  SNPs printed to file \"%s\"", SNPfile);
				fclose(fpSNPs);
				fprintf (stdout, "\n  Haplotypes printed to file \"%s\"", haplotypeFile);
				fclose(fpHaplotypes);
				}
			if (doPrintGenotypes == YES)
				{
				fprintf (stdout, "\n  Genotypes printed to file \"%s\"", genotypeFile);
				fclose(fpGenotypes);
				}
			if (doPrintSequences == YES)
				{
				fprintf (stdout, "\n  Sequences printed to file \"%s\"", sequenceFile);
				fclose(fpSequences);
				}
			if (doPrintTrees == YES)
				{
				fprintf (stdout, "\n  Trees printed to file \"%s\"", treeFile);
				fclose(fpTrees);
				}
			if (doPrintTimes == YES)
				{
				fprintf (stdout, "\n  Times printed to file \"%s\"", timesFile);
				fclose(fpTimes);
				}
			if (doPrintMutationModel == YES)
				{
				fprintf (stdout, "\n  Mutations printed to file \"%s\"\n\n", mutationsFile);
				fclose(fpMutations);
				}
			}
		else	
			fprintf (stdout, "\n\n\nNo output files");

		}
	
	#ifdef CHECK_MUT_DISTRIBUTION
		fprintf (stdout,"\n\nChecking distribution of mutations");
		for (i=1; i<=10; i++)
			fprintf (stdout,"\nSites with %d mutations = %6.4f  (Expected = %6.4f)",i,(double)MutCount[i]/numDataSets, theta/i);
		
		fprintf (stdout, "\nmeanSumPos = %6.4f  (expected (L/2) = %6.4f)\n\n", meanSumPos / (double) dataSetsWithSNPs, numSites/2.0);
	#endif

	secs = (double)(clock() - start) / CLOCKS_PER_SEC;

	if (noisy > 0)
		{
		printf("\n\n_________________________________________________________________");
		printf("\nTime processing: %G seconds\n\n", secs);
		}

	return (1);
}




/************************* MakeCoalescenceTreeHotspotRec  ************************/
/*	Builds a genealogy for each site under the coalescent with or without
	recombination. Recombination distributed in hotspots, so it creates a 
	haplotype block structure. A recombinational event can occur in a hotspot
	(with the higher block rate) or in the rest of the sequence (with the lower homogeneous rate). 
	If the recombination rate occurs in a hotspot, there will be small offset depending
    on a imprecision parameter. This function sets first a pdf for recombination along the 
    gene and then chooses gametes and sites accordingly
	
*/
  
void MakeCoalescenceTreeHotspotRec (int numGametes, int numSites, int N, long int *seed)
{
	int			i, j, *activeGametes, numActive, isCoalescence, whichGamete, 
				whichSite, nextAvailable, hasPassedBreakPoint,
				firstInd, secondInd, newInd, firstHalf, secondHalf, foundSuperflousNode, eventNum,
				lowerTail, upperTail, printRecPDF, period;
	double		rateCA, rateRE, rate, currentTime, timeRE, timeCA, eventTime, probRecIndividual, probRecSite;
	TreeNode	*p, *q, *r;
	double 		G, *g, cum_gi, cum_gamma;
	int			*giBegin, *giEnd;
	double		*gamma, *counts;
	FILE		*fpCounts;
	printRecPDF = NO;
	
	/* allocate space for tree */
	nodes = (TreeNode *) calloc(numSites * numNodes, sizeof(TreeNode));
	if (!nodes)
		{
		fprintf (stderr, "Could not allocate nodes (%d)\n", numSites * numNodes * sizeof(TreeNode));
		exit (1);
		}
	activeGametes = (int *) calloc(numNodes, sizeof(int));
	if (!activeGametes)
		{
		fprintf (stderr, "Could not allocate activeGametes (%d)\n", numNodes * sizeof(int));
		exit (1);
		}
	treeRoot = (TreeNode **) calloc(numSites, sizeof(TreeNode *));
	if (!treeRoot)
		{
		fprintf (stderr, "Could not allocate treeRoot (%d)\n", numSites * sizeof(TreeNode));
		exit (1);
		}
	hotspot = (int *) calloc(numSites + 2*numSitesOutside, sizeof(int));
	if (!hotspot)
		{
		fprintf (stderr, "Could not allocate hotspots (%d)\n", numSites + 2*numSitesOutside * sizeof(int));
		exit (1);
		}
	counts = (double *) calloc(numSites + 2*numSitesOutside, sizeof(double));
	if (!counts)
		{
		fprintf (stderr, "Could not allocate counts (%d)\n", numSites + 2*numSitesOutside * sizeof(int));
		exit (1);
		}
	gamma = (double *) calloc(numSites + 2*numSitesOutside, sizeof(double));
	if (!gamma)
		{
		fprintf (stderr, "Could not allocate gamma (%d)\n", numSites + 2*numSitesOutside * sizeof(double));
		exit (1);
		}

	/* allocate space for MRCA vector = [n1, n2, ..., ni,...nk] 
	   where n is the number of sequences and k is the number of sites
	   When an element of MRCA reaches 1, means that the site i found its MRCA */
		MRCA = (int *) calloc(numSites, sizeof(int));
			if (!MRCA)
			{
			fprintf (stderr, "Could not allocate MRCA (%d)\n", numSites * sizeof(int));
			exit (-1);
			}
			
	/* set everything to null */
	for (i=0; i<numSites*numNodes; i++)
		{
		nodes[i].left = NULL;
		nodes[i].right = NULL;
		nodes[i].anc = NULL;
		nodes[i].time = 0.0;
		nodes[i].length = 0.0;
		nodes[i].index = 0;
		nodes[i].label = 0;
		nodes[i].isAncestralSite = NO;
		}

	/* set up MRCA vector */
	for (i=0; i< numSites; i++)
		MRCA[i] = numGametes; 

	/* set up initial gametes */
	numActive = 0;
	for (i=0; i<numGametes; i++)
		{
		for (j=0; j<numSites; j++)
			{
			p = nodes + pos(i,j,numSites);
			activeGametes[numActive] = i;
			p->isAncestralSite = YES;
			p->index = i;
			}
		numActive++;
		}
	nextAvailable = numActive;
		
	/* print out ancestral (active) status for each site */
	if (noisy > 3)
		{
		for (i=0; i<numActive; i++)
			{
			for (j=0; j<numSites; j++)
				{
				p = nodes + pos(activeGametes[i],j,numSites);
				if (j == 0)
					fprintf (stderr, "%4d -- ", p->index+1);
				fprintf (stderr, "%d", p->isAncestralSite);
				}
			fprintf (stderr, "\n");
			}
		}	

	/*	(1) If the user has fixed the number of hotspots locate them uniformly
			otherwise get number of hotspots from Poisson anf their positions from
			an Exponential (no interference) or Gamm (interference)  */
	if (fixedNumHotspots == 0)	
		numHotspots = GammaHotspots (expNumPoissonHotspots * (numSites + 2*numSitesOutside) / (double) numSites, interferenceInterval, seed);
	else 
		{
		numHotspots = fixedNumHotspots;
		UniformHotspots (numHotspots, seed);
		}
	
	/* (2) Simulate the recombination PDF along the gene and extra tails */
	SimRecombinationPDF (counts, seed);
	
/*
	fprintf (stderr,"\n%d hotspots BEFORE = ", numHotspots);
	for (i=0; i<numHotspots; i++)
		fprintf (stderr, "%d ", hotspot[i]+1);
*/

	/* (3) If recombinations were simulated, was used get rid of tails E outside gene: correct counts and hotspots 
		We are taking into account offshots from hotspots situated some 
		numSitesOutside (E) the gene (L) 

	      E          L            E
		-----0000000000000000000-----  */

	if (printRecPDF == YES)
		{
		 fpCounts = fopen ("RecPDF1", "w+");
		if (fpCounts == NULL)
			{
			fprintf (stderr, "Could not open the \"RecPDF1\" file");
			exit (1);
			}
		fprintf (fpCounts, "\n\n\n\nBEFORE relabelling of counts and hotspots");
		for (i=0; i<numSites+2*numSitesOutside; i++)
			fprintf (fpCounts, "\n%04d %6.4f", i+1, counts[i]);
		fclose (fpCounts);
		}
	
	if (fixedNumHotspots == 0)	
		{
		/* correct counts */
		for (i=0; i<numSites; i++)
			counts[i] = counts[i+numSitesOutside];

		/* in case there are hotspots outside, relabel them */
		lowerTail = upperTail = 0;
		for (i=0; i<numHotspots; i++)
			{
			if (hotspot[i] < numSitesOutside)
				lowerTail++;
			else if (hotspot[i] >= numSites + numSitesOutside)
				upperTail++;
			}
		numHotspots -= (lowerTail + upperTail);
		for (i=0; i<numHotspots; i++)
			hotspot[i] = hotspot[i+lowerTail] - numSitesOutside;
		}
	
/*
	fprintf (stderr,"\n%d hotspots AFTER (lowerTail = %d  upperTail = %d) = ", numHotspots, lowerTail, upperTail);
		for (i=0; i<numHotspots; i++)
			fprintf (stderr, "%d ", hotspot[i]+1);
*/
	if (printRecPDF == YES)
		{
		 fpCounts = fopen ("RecPDF2", "w+");
		if (fpCounts == NULL)
			{
			fprintf (stderr, "Could not open the \"RecPDF2\" file");
			exit (1);
			}
		fprintf (fpCounts, "\n\n\n\nAFTER relabelling of counts and hotspots");
		for (i=0; i<numSites; i++)
			fprintf (fpCounts, "\n%04d %6.4f", i+1, counts[i]);
		fclose (fpCounts);
		}
		
	/* set global recombination rate per site (gamma) */
	for (i=0; i<numSites; i++)
		{
		gamma[i] = recombinationRate + blockRecombinationRate * ((double) counts[i] / (double) numPointsRecPDF);	
		/* fprintf (stderr, "gamma[i]= %10.8f\n", gamma[i]);*/
		}

	/* calculate overall recombination rate = integrate over gamma PDF */
	C01 = 0;
	for (i=0; i<numSites; i++)
		C01 += gamma[i];
	C01 *= 4.0 * N;
	
	/* make coalescence tree */	
	eventNum = 0;		
	currentTime = 0.0;
	period = 1;
	while (numActive > 1)
		{
		/* print out MRCA vector */
		if (noisy > 6)
			{	
			fprintf (stderr,"MRCA [] = ");
			for (j=0; j<numSites; j++)
				fprintf (stderr,"%d ", MRCA[j]);
			fprintf (stderr,"\n");
			}
		/* allocate memory for each individual g and its limits */
		g = (double *) calloc(numActive, sizeof(double));
			if (!g)
			{
			fprintf (stderr, "Could not allocate g (%d)\n", numActive * sizeof(double));
			exit (-1);
			}
		giBegin = (int *) calloc(numActive, sizeof(int));
			if (!giBegin)
			{
			fprintf (stderr, "Could not allocate giBegin (%d)\n", numActive * sizeof(int));
			exit (-1);
			}
		giEnd = (int *) calloc(numActive, sizeof(int));
			if (!giEnd)
			{
			fprintf (stderr, "Could not allocate giEnd (%d)\n", numActive * sizeof(int));
			exit (-1);
			}
		
		/* calculate valid segments */	
		for (i=0; i<numActive; i++)
			CalcIndividualG (i, giBegin, giEnd, numSites, nodes, activeGametes);
			
		/* calculate g[i] and G */
		G = 0;
		for (i=0; i<numActive; i++)
			{
			g[i] = 0;
			for (j=0; j<numSites; j++)
				{
				/*if (IsValidBreakSite (activeGametes, i, j, nodes) == YES)*/
				if (j >= giBegin[i] && j <= giEnd[i])
					g[i] += gamma[j];
				}
			G += g[i];
			if (noisy > 4)
				fprintf (stderr,"%f (%d,%d) ", g[i], giBegin[i]+1, giEnd[i]+1);
			}
			
		if (noisy > 4)
			fprintf (stderr,"  G = %f ", G);
		
		/* choose event; time is kept in generations (scaled by 2N)  */  
		rateRE = 2.0 * N * G;
		rateCA = numActive * (numActive - 1) / 2.0;
		rate =  rateCA + rateRE;

		if (doDemographics == YES)
			{
			periodGrowth[period] = -log (Nend[period] / (double) Nbegin[period]) / (double) (cumDuration[period] - cumDuration[period-1]);
			if (isnan(periodGrowth[period]) == YES)
				{
				fprintf (stderr, "\nERROR: period growth (%f) is NaN", periodGrowth[period]);
				fprintf (stderr, "\n       This might suggest that the growth rate is too negative");
				fprintf (stderr, "\n       and the coalescent time is therefore infinite.");
				fprintf (stderr, "\n       Try a smaller value");
				exit (1);
				}

			timeCA =  log (1 + RandomExponential (rateCA, seed) * periodGrowth[period] * 2. * Nbegin[period] * 
			                exp (-periodGrowth[period] * (currentTime - cumDuration[period-1]))) / periodGrowth[period];

			/*	When growth rate is very negative, coalescent time may be infinite
				this results in log (-x) => timCA = NaN. If this not the last period
				just jump to the next. If this is the last period, we have to exit
				the program
			*/
			if (isnan(timeCA) == YES)
				{
				if (period < numPeriods) 
					{
					currentTime = cumDuration[period];
					period++;
					periodGrowth[period] = -log (Nend[period] / (double) Nbegin[period]) / (double) cumDuration[period];
					continue;
					}
				else
					{
					fprintf (stderr, "\nERROR: Coalescent time (%f) is infinite ", timeCA);
					fprintf (stderr, "\n       This might suggest that the growth rate is too negative");
					fprintf (stderr, "\n       and the coalescent time is therefore infinite.");
					fprintf (stderr, "\n       Try a smaller value");
					exit (1);
					}
				}
			}
		else
			{
			timeCA = RandomExponential (rateCA, seed) * 2.0 * N;
			if (doExponential == YES)
				timeCA = log (exp(growthRate*currentTime) + growthRate * timeCA) / growthRate - currentTime;
		
			/*	When growth rate is very negative, coalescent time may be infinite
				this results in log (-x) => timCA = NaN. We have to exit
				the program
			*/
			if (isnan(timeCA) == YES)
				{
				fprintf (stderr, "\nERROR: Coalescent time (%f) is infinite ", timeCA);
				fprintf (stderr, "\n       This might suggest that the growth rate is too negative");
				fprintf (stderr, "\n       and the coalescent time is therefore infinite.");
				fprintf (stderr, "\n       Try a smaller value");
				exit (1);
				}
			}

		timeRE = RandomExponential (rateRE, seed) * 2.0 * N;
	
		/* (4) event is a coalescence or a recombination? */
		if (timeCA < timeRE)
			{
			isCoalescence = YES;
			eventTime = timeCA;		
			/*	if this period is not the last one and if the event time is outside the current interval,
				update period and start again */
			if (doDemographics == YES && period < numPeriods  &&  (currentTime + eventTime) > cumDuration [period])
				{
				currentTime = cumDuration[period];
				period++;
				periodGrowth[period] = -log (Nend[period] - Nbegin[period]) / (double) cumDuration[period];
				continue;
				}	
			numCA++;
			}
		else
			{
			isCoalescence = NO;
			numRE++;
			eventTime = timeRE;	
			}

		/* (5) set time */
		currentTime += eventTime;                              

		if (noisy > 2)
			fprintf (stderr, "\n\nEvent %3d :  rate = %lf, currentTime = %lf\n", ++eventNum, rate, currentTime);

		
		/* if RECOMBINATION, readjust active gametes */
		if (isCoalescence == NO)
			{

			/* (6) Choose which gamete has the recombination based on their g values */
			cum_gi = 0;
			probRecIndividual = G * RandomUniform(seed); 
			for (whichGamete=0; whichGamete<numActive; whichGamete++)
				{
				cum_gi += g[whichGamete];
				if (cum_gi > probRecIndividual)
					break;
				}
			if (whichGamete >= numActive)
				{
				fprintf (stderr, "\n\nERROR: whichGamete (%d) out of range!  G = %f  probRecInd = %f\n", 
				                whichGamete, G, probRecIndividual);
				exit (-1);
				}

			/* (7) Choose whichSite based on gammma[j] */
			cum_gamma = 0;
			probRecSite = g[whichGamete] * RandomUniform(seed); 
			for (whichSite=giBegin[whichGamete]; whichSite<=giEnd[whichGamete]; whichSite++)
				{
				cum_gamma += gamma[whichSite];
				if (cum_gamma > probRecSite)
					break;
				}
			if (whichSite >= numSites)
				{
				fprintf (stderr, "\n\nERROR: whichSite (%d) out of range!\n", whichSite+1);
				exit (-1);
				}
		
	
			/* should this recombination event be counted in the expected number of recombinations E(R)? */
			/* for E(R) count only events with breakpoints as 1|1, 1|0 or 0|1   (i.e., not 0|0)          */
			/* if 1 represent a site that did found already its MRCA count it as a 0 */
			if (CountsForExpNumRec (activeGametes, whichGamete, whichSite, nodes, MRCA) == NO)
				{
				recNotToCount++;
				/* fprintf (stderr,"..not to count.."); */
				}

			/* copy whichIndividual to a new space in memory */
			hasPassedBreakPoint = NO;
			firstHalf = nextAvailable++;
			if (nextAvailable >= numNodes)
				{
				fprintf (stderr, "\n\nERROR: Too many nodes!\n");
				exit (-1);
				}
			secondHalf = nextAvailable++;
			if (nextAvailable >= numNodes)
				{
				fprintf (stderr, "\n\nERROR: Too many nodes!\n");
				exit (-1);
				}
			for (j=0; j<numSites; j++)
				{
				if (j > whichSite)
					hasPassedBreakPoint = YES;
				p = nodes + pos(activeGametes[whichGamete],j,numSites);
				q = nodes + pos(firstHalf,j,numSites);
				r = nodes + pos(secondHalf,j,numSites);
				q->index = firstHalf;
				r->index = secondHalf;
				if (p->isAncestralSite == NO)
					{
					q->isAncestralSite = NO;
					r->isAncestralSite = NO;
					}
				else
					{
					if (hasPassedBreakPoint == NO)
						{
						q->isAncestralSite = YES;
						r->isAncestralSite = NO;
						q->left = p; 
						p->anc = q; 
						}
					else
						{
						q->isAncestralSite = NO;
						r->isAncestralSite = YES;
						r->left = p; 
						p->anc = r;
						}
					}
				}

			if (noisy > 2)
				{
				fprintf (stderr, "  Recombination involving %d (copied to %d and %d)", p->index+1, q->index+1, r->index+1 );
				fprintf (stderr, "\n    breakpoint at site %d\n", whichSite+1);
				}

			/* readjust active sites */
			activeGametes[whichGamete] = firstHalf;
			activeGametes[numActive++] = secondHalf;
			}
		
		/* if COALESCENCE, readjust individuals and pointers */
		if (isCoalescence == YES)
			{

			/* figure out which two individuals are involved */
			firstInd = numActive * RandomUniform(seed);
			if (firstInd >= numActive)
				{
				fprintf (stderr, "\n\nERROR: firstInd out of range!\n");
				exit (-1);
				}
			do
				{
				secondInd = numActive * RandomUniform(seed);
				} while (firstInd == secondInd);
			newInd = nextAvailable;
			if (noisy > 2)
				fprintf (stderr, "  Coalescence involving %d and %d to create gamete %d\n", 
				activeGametes[firstInd]+1, activeGametes[secondInd]+1, newInd+1);

			/* readjust MRCA, pointers and active sites */
			for (j=0; j<numSites; j++)
				{
				p = nodes + pos(activeGametes[firstInd],j,numSites);
				q = nodes + pos(activeGametes[secondInd],j,numSites);
				r = nodes + pos(newInd,j,numSites);
				r->index = nextAvailable;
				if (p->isAncestralSite == YES && q->isAncestralSite == YES)
					MRCA[j]--;
				if (p->isAncestralSite == YES || q->isAncestralSite == YES)
					r->isAncestralSite = YES;
				else
					r->isAncestralSite = NO;
				if (p->isAncestralSite == YES && q->isAncestralSite == YES )
					{
					r->left = p;
					r->right = q;
					p->anc = r;
					q->anc = r;
					r->time = currentTime;
					}
				else if (p->isAncestralSite == YES && q->isAncestralSite == NO)
					{
					r->left = p;
					p->anc = r;
					}
				else if (p->isAncestralSite == NO && q->isAncestralSite == YES)
					{
					r->left = q;
					q->anc = r;
					}

				}
			/* readjust active gametes */
			activeGametes[firstInd] = newInd;
			activeGametes[secondInd] = activeGametes[numActive-1];
			numActive--;
			nextAvailable++;
			if (nextAvailable >= numNodes)
				{
				fprintf (stderr, "\n\nERROR: Too many nodes!\n");
				exit (-1);
				}
			}

		/* print out ancestral (active) status for each site and MRCA vector */
		if (noisy > 3)
			{
			for (i=0; i<numActive; i++)
				{
				for (j=0; j<numSites; j++)
					{
					p = nodes + pos(activeGametes[i],j,numSites);
					if (j == 0)
						fprintf (stderr, "%4d -- ", p->index+1);
					fprintf (stderr, "%d", p->isAncestralSite);
					}
				fprintf (stderr, "\n");
				}
			fprintf (stderr, "MRCA    ");
			for (j=0; j<numSites; j++)
				{
				if (MRCA[j] <= 1)
					fprintf (stderr, "*");
				else
					fprintf (stderr, " ");
				}
			fprintf (stderr, "\n");
			
			
			if (doBlockRecombination == YES)
				{
				fprintf (stderr, "Hotspot ");
				i = 0;
				for (j=0; j<numSites; j++)
					{
					if (hotspot[i] == j)
						{
						fprintf (stderr, "+");
						i++;
						}
					else
						fprintf (stderr, " ");
					}
				fprintf (stderr, "\n");
				}
			
			
			}

		} /* coalescent tree finished */

	TMRCA = currentTime / (2.0 * N);

	if (noisy > 2)
		{
		fprintf (stderr, "\n>> Coalescent tree finished\n");
		fprintf (stderr, "\n>> Start to get rid of superflous nodes... \n");
		}		

	/* get rid of superflous nodes */
	foundSuperflousNode = YES;
	while (foundSuperflousNode == YES)
		{
		foundSuperflousNode = NO;
		for (i=0; i<nextAvailable; i++)
			{
			for (j=0; j<numSites; j++)
				{
				p = nodes + pos(i,j,numSites);
				if (p->left == NULL && p->right == NULL && p->anc == NULL)
					{
					/* nothing to do with this node because it is not connected to anything */
					}
				else if (p->left == NULL && p->right == NULL && p->anc != NULL)
					{
					/* do not do anything with this node because it is a tip */
					}
				else if (p->left != NULL && p->right == NULL && p->anc != NULL)
					{
					/* this is a superflous node and can be removed */
					foundSuperflousNode = YES;
					q = p->left;
					r = p->anc;
					if (p->anc->left == p)
						{
						r->left = q;
						q->anc = r;
						p->left = NULL;
						p->anc = NULL;
						}
					else
						{
						r->right = q;
						q->anc = r;
						p->left = NULL;
						p->anc = NULL;
						}
					}
				else if (p->left == NULL && p->right != NULL && p->anc != NULL)
					{
					/* this is a superflous node and can be removed */
					foundSuperflousNode = YES;
					q = p->right;
					r = p->anc;
					if (p->anc->left == p)
						{
						r->left = q;
						q->anc = r;
						p->right = NULL;
						p->anc = NULL;
						}
					else
						{
						r->right = q;
						q->anc = r;
						p->right = NULL;
						p->anc = NULL;
						}
					}
				else if (p->left != NULL && p->right != NULL && p->anc != NULL)
					{
					/* this is an internal node formed by a coalescence event, do not touch */
					}
				else if (p->left != NULL && p->right != NULL && p->anc == NULL)
					{
					/* this is the last (coalescence event) in the tree */
					}
				else if (p->left != NULL && p->right == NULL && p->anc == NULL)
					{
					/* Seems to be the last coalescent event among sequences with non-ancestral material */	
					/* it is not superfluous, we just remove it */
					p->left->anc = NULL;
					}
				else if (p->left == NULL && p->right != NULL && p->anc == NULL)
					{
					/* not clear what this node could be doing, but we will remove it anyway */
					fprintf (stderr, "strange\n");
					p->left = NULL;
					}
				else
					{
					fprintf (stderr, "You should not be here, I think\n");
					fprintf (stderr, "%d %d-- %d %d %d\n", Index(p), j, Index(p->left), Index(p->right), Index(p->anc)); 
					}
				}
			}
		}
	
	/* set up root pointers */
	for (j=0; j<numSites; j++)
		{
		for (i=0; i<nextAvailable; i++)
			{
			p = nodes + pos(i,j,numSites);
			if (p->left != NULL && p->right != NULL && p->anc == NULL)
				treeRoot[j] = p;
			}
		}

	/* Relabel nodes on tree (label tips with indexes) */
	if (noisy > 2)
		fprintf (stderr, "\n>> Relabeling nodes on tree... \n");
	for (j=0; j<numSites; j++)
		{
		tipLabel = 0;
		intLabel = numGametes;
		RelabelNodes(treeRoot[j]);
		}

	free (activeGametes);

}





/************************* MakeCoalescenceTreeHomoRec  ************************/
/*	Builds a genealogy for each site under the coalescent with or without
	recombination. Recombination is only homogeneously distributed       */
 
void MakeCoalescenceTreeHomoRec (int numGametes, int numSites, int N, long int *seed)
{
	int			i, j, *activeGametes, numActive, isCoalescence, whichGamete, 
				whichSite, nextAvailable, hasPassedBreakPoint,
				firstInd, secondInd, newInd, firstHalf, secondHalf, foundSuperflousNode, eventNum,
				probRecIndividual, cum_gi, period;
	double		rateCA, rateRE, rate, timeRE, timeCA, currentTime, eventTime;
	TreeNode	*p, *q, *r;
	long int 	G;
	int			*g, *giBegin, *giEnd;

	/* allocate space for tree */
	nodes = (TreeNode *) calloc(numSites * numNodes, sizeof(TreeNode));
	if (!nodes)
		{
		fprintf (stderr, "Could not allocate nodes (%d)\n", numSites * numNodes * sizeof(TreeNode));
		exit (1);
		}
	activeGametes = (int *) calloc(numNodes, sizeof(int));
	if (!activeGametes)
		{
		fprintf (stderr, "Could not allocate activeGametes (%d)\n", numNodes * sizeof(int));
		exit (1);
		}
	treeRoot = (TreeNode **) calloc(numSites, sizeof(TreeNode *));
	if (!treeRoot)
		{
		fprintf (stderr, "Could not allocate treeRoot (%d)\n", numSites * sizeof(TreeNode));
		exit (1);
		}

	/* allocate space for MRCA vector = [n1, n2, ..., ni,...nk] 
	   where n is the number of sequences and k is the number of sites
	   When an element of MRCA reaches 1, means that the site i found its MRCA */
	   
	MRCA = (int *) calloc(numSites, sizeof(int));
	if (!MRCA)
		{
		fprintf (stderr, "Could not allocate MRCA (%d)\n", numSites * sizeof(int));
		exit (-1);
		}

	/* set everything to null */
	for (i=0; i<numSites*numNodes; i++)
		{
		nodes[i].left = NULL;
		nodes[i].right = NULL;
		nodes[i].anc = NULL;
		nodes[i].time = 0.0;
		nodes[i].length = 0.0;
		nodes[i].index = 0;
		nodes[i].label = 0;
		nodes[i].isAncestralSite = NO;
		}

	/* set up MRCA vector */
	for (i=0; i< numSites; i++)
		MRCA[i] = numGametes; 

	/* set up initial gametes */
	numActive = 0;
	for (i=0; i<numGametes; i++)
		{
		for (j=0; j<numSites; j++)
			{
			p = nodes + pos(i,j,numSites);
			activeGametes[numActive] = i;
			p->isAncestralSite = YES;
			p->index = i;
			}
		numActive++;
		}
	nextAvailable = numActive;
		
	/* print out ancestral (active) status for each site */
	if (noisy > 3)
		{
		for (i=0; i<numActive; i++)
			{
			for (j=0; j<numSites; j++)
				{
				p = nodes + pos(activeGametes[i],j,numSites);
				if (j == 0)
					fprintf (stderr, "%4d -- ", p->index+1);
				fprintf (stderr, "%d", p->isAncestralSite);
				}
			fprintf (stderr, "\n");
			}
		}	

	/* make coalescence tree */	
	eventNum = 0;		
	period = 1;
	currentTime = 0.0;
	eventTime = 0.0;
	while (numActive > 1)
		{
		/* print out MRCA vector */
		if (noisy > 6)
			{	
			fprintf (stderr,"MRCA [] = ");
			for (j=0; j<numSites; j++)
					fprintf (stderr,"%d ", MRCA[j]);
			fprintf (stderr,"\n");
			}
		G = 0;
		/* allocate memory for each individual g and its limits */
		g = (int *) calloc(numActive, sizeof(int));
			if (!g)
			{
			fprintf (stderr, "Could not allocate g (%d)\n", numActive * sizeof(int));
			exit (-1);
			}
		giBegin = (int *) calloc(numActive, sizeof(int));
			if (!giBegin)
			{
			fprintf (stderr, "Could not allocate giBegin (%d)\n", numActive * sizeof(int));
			exit (-1);
			}
		giEnd = (int *) calloc(numActive, sizeof(int));
			if (!giEnd)
			{
			fprintf (stderr, "Could not allocate giEnd (%d)\n", numActive * sizeof(int));
			exit (-1);
			}
		/* calculate g for each gamete, its limits, and total G */	
		for (i=0; i<numActive; i++)
			{	
			g[i] = CalcIndividualG (i, giBegin, giEnd, numSites, nodes, activeGametes);
			G += g[i];
			if (noisy > 4)
				fprintf (stderr,"%d (%d,%d) ", g[i], giBegin[i]+1, giEnd[i]+1);
			}	
		if (noisy > 4)
			fprintf (stderr,"  G = %d ", G);


		/* choose event; time is kept in generations (scaled by 2N)  */  
		rateRE = G * 2.0 * N * recombinationRate;
		rateCA = numActive * (numActive - 1) / 2.0;
		rate =  rateCA + rateRE;

		if (doDemographics == YES)
			{
			periodGrowth[period] = -log (Nend[period] / (double) Nbegin[period]) / (double) (cumDuration[period] - cumDuration[period-1]);
			if (isnan(periodGrowth[period]) == YES)
				{
				fprintf (stderr, "\nERROR: period growth (%f) is NaN", periodGrowth[period]);
				fprintf (stderr, "\n       This might suggest that the growth rate is too negative");
				fprintf (stderr, "\n       and the coalescent time is therefore infinite.");
				fprintf (stderr, "\n       Try a smaller value");
				exit (1);
				}

			timeCA =  log (1 + RandomExponential (rateCA, seed) * periodGrowth[period] * 2. * Nbegin[period] * 
			                exp (-periodGrowth[period] * (currentTime - cumDuration[period-1]))) / periodGrowth[period];

			/*	When growth rate is very negative, coalescent time may be infinite
				this results in log (-x) => timCA = NaN. If this not the last period
				just jump to the next. If this is the last period, we have to exit
				the program
			*/
			if (isnan(timeCA) == YES)
				{
				if (period < numPeriods) 
					{
					currentTime = cumDuration[period];
					period++;
					periodGrowth[period] = -log (Nend[period] / (double) Nbegin[period]) / (double) cumDuration[period];
					continue;
					}
				else
					{
					fprintf (stderr, "\nERROR: Coalescent time (%f) is infinite ", timeCA);
					fprintf (stderr, "\n       This might suggest that the growth rate is too negative");
					fprintf (stderr, "\n       and the coalescent time is therefore infinite.");
					fprintf (stderr, "\n       Try a smaller value");
					exit (1);
					}
				}
			}
		else
			{
			timeCA = RandomExponential (rateCA, seed) * 2.0 * N;
			if (doExponential == YES)
				{
				timeCA = log (exp(growthRate*currentTime) + growthRate * timeCA) / growthRate - currentTime;
		
				/*	When growth rate is very negative, coalescent time may be infinite
					this results in log (-x) => timCA = NaN. We have to exit
					the program
				*/
				if (isnan(timeCA) == YES)
					{
					fprintf (stderr, "\nERROR: Coalescent time (%f) is infinite ", timeCA);
					fprintf (stderr, "\n       This might suggest that the growth rate is too negative");
					fprintf (stderr, "\n       and the coalescent time is therefore infinite.");
					fprintf (stderr, "\n       Try a smaller value");
					exit (1);
					}
				}
			else if (doLogistic == YES) 
				{
				/* we have not implemented this yet */
				}
			}

		timeRE = RandomExponential (rateRE, seed) * 2.0 * N;

		/*  event is a coalescence or a recombination? */
		if (timeCA < timeRE)
			{
			isCoalescence = YES;
			eventTime = timeCA;		
			/*	if this period is not the last one and if the event time is outside the current interval,
				update period and start again */
			if (doDemographics == YES && period < numPeriods  &&  (currentTime + eventTime) > cumDuration [period])
				{
				currentTime = cumDuration[period];
				period++;
				periodGrowth[period] = -log (Nend[period] - Nbegin[period]) / (double) cumDuration[period];
				continue;
				}	
			numCA++;
			}
		else
			{
			isCoalescence = NO;
			numRE++;
			eventTime = timeRE;	
			}

		/* set time */
		currentTime += eventTime;                              

		if (noisy > 2)
			fprintf (stderr, "\n\nEvent %3d :  rate = %lf, currentTime = %lf\n", ++eventNum, rate, currentTime);
		
		/* if RECOMBINATION, readjust active gametes */
		if (isCoalescence == NO)
			{
			/* Which gamete has the recombination? */
				/* assign probability to each gamete based on their g's values */
			cum_gi = 0;
			probRecIndividual = G * RandomUniform(seed); 
			for (whichGamete=0; whichGamete<numActive; whichGamete++)
				{
				cum_gi += g[whichGamete];
				if (probRecIndividual < cum_gi)
					break;
				}
			if (whichGamete >= numActive)
				{
				fprintf (stderr, "\n\nERROR: whichGamete out of range! : whichGamete = %d\n", whichGamete);
				exit (-1);
				}

			/* select a valid breakpoint among potential recombining locations */
			whichSite = giBegin[whichGamete] + g[whichGamete] * RandomUniform(seed);
			
			/* should this recombination event be counted in the expected numebr of recombinations E(R)? */
			/* for E(R) count only events with breakpoints as 1|1, 1|0 or 0|1   (i.e., not 0|0)          */
			/* if 1 represent a site that did found already its MRCA count it as a 0 */
			if (CountsForExpNumRec (activeGametes, whichGamete, whichSite, nodes, MRCA) == NO)
				{
				recNotToCount++;
				/* fprintf (stderr,"..not to count.."); */
				}

			/* copy whichIndividual to a new space in memory */
			hasPassedBreakPoint = NO;
			firstHalf = nextAvailable++;
			if (nextAvailable >= numNodes)
				{
				fprintf (stderr, "\n\nERROR: Too many nodes!\n");
				exit (-1);
				}
			secondHalf = nextAvailable++;
			if (nextAvailable >= numNodes)
				{
				fprintf (stderr, "\n\nERROR: Too many nodes!\n");
				exit (-1);
				}
			for (j=0; j<numSites; j++)
				{
				if (j > whichSite)
					hasPassedBreakPoint = YES;
				p = nodes + pos(activeGametes[whichGamete],j,numSites);
				q = nodes + pos(firstHalf,j,numSites);
				r = nodes + pos(secondHalf,j,numSites);
				q->index = firstHalf;
				r->index = secondHalf;
				if (p->isAncestralSite == NO)
					{
					q->isAncestralSite = NO;
					r->isAncestralSite = NO;
					}
				else
					{
					if (hasPassedBreakPoint == NO)
						{
						q->isAncestralSite = YES;
						r->isAncestralSite = NO;
						q->left = p; 
						p->anc = q; 
						}
					else
						{
						q->isAncestralSite = NO;
						r->isAncestralSite = YES;
						r->left = p; 
						p->anc = r;
						}
					}
				if (j == 0 	&& noisy > 2)
					{
					fprintf (stderr, "  Recombination involving %d (copied to %d and %d)", p->index+1, q->index+1, r->index+1 );
					fprintf (stderr, "\n    breakpoint at site %d\n", whichSite+1);
					}
				}
			/* readjust active sites */
			activeGametes[whichGamete] = firstHalf;
			activeGametes[numActive++] = secondHalf;
			}
		
		/* if COALESCENCE, readjust individuals and pointers */
		if (isCoalescence == YES)
			{

			/* figure out which two individuals are involved */
			firstInd = numActive * RandomUniform(seed);
			if (firstInd >= numActive)
				{
				fprintf (stderr, "\n\nERROR: firstInd out of range!\n");
				exit (-1);
				}
			do
				{
				secondInd = numActive * RandomUniform(seed);
				} while (firstInd == secondInd);
			newInd = nextAvailable;
			if (noisy > 2)
				fprintf (stderr, "  Coalescence involving %d and %d to create gamete %d\n", 
				activeGametes[firstInd]+1, activeGametes[secondInd]+1, newInd+1);

			/* readjust MRCA, pointers and active sites */
			for (j=0; j<numSites; j++)
				{
				p = nodes + pos(activeGametes[firstInd],j,numSites);
				q = nodes + pos(activeGametes[secondInd],j,numSites);
				r = nodes + pos(newInd,j,numSites);
				r->index = nextAvailable;
				if (p->isAncestralSite == YES && q->isAncestralSite == YES)
					MRCA[j]--;
				if (p->isAncestralSite == YES || q->isAncestralSite == YES)
					r->isAncestralSite = YES;
				else
					r->isAncestralSite = NO;
				if (p->isAncestralSite == YES && q->isAncestralSite == YES )
					{
					r->left = p;
					r->right = q;
					p->anc = r;
					q->anc = r;
					r->time = currentTime;
					}
				else if (p->isAncestralSite == YES && q->isAncestralSite == NO)
					{
					r->left = p;
					p->anc = r;
					}
				else if (p->isAncestralSite == NO && q->isAncestralSite == YES)
					{
					r->left = q;
					q->anc = r;
					}
				}
			/* readjust active gametes */
			activeGametes[firstInd] = newInd;
			activeGametes[secondInd] = activeGametes[numActive-1];
			numActive--;
			nextAvailable++;
			if (nextAvailable >= numNodes)
				{
				fprintf (stderr, "\n\nERROR: Too many nodes!\n");
				exit (-1);
				}
			}

		/* print out ancestral (active) status for each site and MRCA vector */
		if (noisy > 3)
			{
			for (i=0; i<numActive; i++)
				{
				for (j=0; j<numSites; j++)
					{
					p = nodes + pos(activeGametes[i],j,numSites);
					if (j == 0)
						fprintf (stderr, "%4d -- ", p->index+1);
					fprintf (stderr, "%d", p->isAncestralSite);
					}
				fprintf (stderr, "\n");
				}
			fprintf (stderr, "MRCA    ");
			for (j=0; j<numSites; j++)
				{
				if (MRCA[j] <= 1)
					fprintf (stderr, "*");
				else
					fprintf (stderr, " ");
				}
			fprintf (stderr, "\n");
			}

		free (g);
		free (giBegin);
		free (giEnd);
		} /* coalescent tree finished */

	TMRCA = currentTime / (2.0 * N);

	if (noisy > 2)
		{
		fprintf (stderr, "\n>> Coalescent tree finished\n");
		fprintf (stderr, "\n>> Start to get rid of superflous nodes... \n");
		}		

	/* get rid of superflous nodes */
	foundSuperflousNode = YES;
	while (foundSuperflousNode == YES)
		{
		foundSuperflousNode = NO;
		for (i=0; i<nextAvailable; i++)
			{
			for (j=0; j<numSites; j++)
				{
				p = nodes + pos(i,j,numSites);
				if (p->left == NULL && p->right == NULL && p->anc == NULL)
					{
					/* nothing to do with this node because it is not connected to anything */
					}
				else if (p->left == NULL && p->right == NULL && p->anc != NULL)
					{
					/* do not do anything with this node because it is a tip */
					}
				else if (p->left != NULL && p->right == NULL && p->anc != NULL)
					{
					/* this is a superflous node and can be removed */
					foundSuperflousNode = YES;
					q = p->left;
					r = p->anc;
					if (p->anc->left == p)
						{
						r->left = q;
						q->anc = r;
						p->left = NULL;
						p->anc = NULL;
						}
					else
						{
						r->right = q;
						q->anc = r;
						p->left = NULL;
						p->anc = NULL;
						}
					}
				else if (p->left == NULL && p->right != NULL && p->anc != NULL)
					{
					/* this is a superflous node and can be removed */
					foundSuperflousNode = YES;
					q = p->right;
					r = p->anc;
					if (p->anc->left == p)
						{
						r->left = q;
						q->anc = r;
						p->right = NULL;
						p->anc = NULL;
						}
					else
						{
						r->right = q;
						q->anc = r;
						p->right = NULL;
						p->anc = NULL;
						}
					}
				else if (p->left != NULL && p->right != NULL && p->anc != NULL)
					{
					/* this is an internal node formed by a coalescence event, do not touch */
					}
				else if (p->left != NULL && p->right != NULL && p->anc == NULL)
					{
					/* this is the last (coalescence event) in the tree */
					}
				else if (p->left != NULL && p->right == NULL && p->anc == NULL)
					{
					/* Seems to be the last coalescent event among sequences with non-ancestral material */	
					/* it is not superfluous, we just remove it */
					p->left->anc = NULL;
					}
				else if (p->left == NULL && p->right != NULL && p->anc == NULL)
					{
					/* not clear what this node could be doing, but we will remove it anyway */
					fprintf (stderr, "We should not be here, strange\n");
					p->left = NULL;
					}
				else
					{
					fprintf (stderr, "We should not be here, I think\n");
					fprintf (stderr, "%d %d-- %d %d %d\n", Index(p), j, Index(p->left), Index(p->right), Index(p->anc)); 
					}
				}
			}
		}

	/* set up root pointers */
	for (j=0; j<numSites; j++)
		{
		for (i=0; i<nextAvailable; i++)
			{
			p = nodes + pos(i,j,numSites);
			if (p->left != NULL && p->right != NULL && p->anc == NULL)
				treeRoot[j] = p;
			}
		}

	/* Relabel nodes on tree (label tips with indexes) */
	if (noisy > 2)
		fprintf (stderr, "\n>> Relabeling nodes on tree... \n");
	for (j=0; j<numSites; j++)
		{
		tipLabel = 0;
		intLabel = numGametes;
		RelabelNodes(treeRoot[j]);
		}

	free (activeGametes);

}





/********************************** EvolveHaplotypesOnTree ***********************************/
/* Evolves haplotypes on the coalescent trees */

void EvolveHaplotypesOnTree (long int *seed, double m, int numSites)
{
	int			i;
	
/* First assign mutation model to each site */
	numJCsites = 0;
	numISMsites = 0;
	for (i=0; i<numSites; i++)
		{
		if (RandomUniform (seed) < propJCsites)
			{
			site[i].isJC = YES;
			numJCsites++;
			}
		else
			{
			site[i].isJC = NO;	
			numISMsites++;
			}
		}
	
	/* Evolve JC sites */
	if (numJCsites > 0)
		for (i=0; i<numSites; i++)
			{
			if (site[i].isJC == YES)
				SimulateJCForSite (treeRoot[i], i, numSites, m*JCrelMutationRate, seed);
			}	
		
	/* Evolve ISM sites */
	if (numISMsites > 0)
		SimulateISM (seed, numISMsites);
		
}




/*************************** SimulateISM **********************************/
/*	Simulates mutations under an infinite sites model (ISM). The number of 
	mutations will be distributed as a Poisson with parameter the sum of
	branch lenghts over all sites.  Times are already scaled in
	2N.  We will force it to get just one mutation per site.

	The above approach, which conditions on theta and takes in account 
	the branch lengths, is preferred,than Hudson's, which selects per each
	site a random branch and puts a mutation there, and therefore it is 
	conditioned on the number of segregating sites. With the coalescent 
	you get large or short trees, which imply different number of mutations
	but if you do Hudson's approach you always put the same number of mutations. 
*/

void SimulateISM (long int *seed, int numISMsites)
{
	int		i, numMutations, mutations;
	double	totalBranchSum, uniform, cumBranch;
	

	/* Sum all branch lenghts at all ISM sites */
	totalBranchSum = 0;
	for (i=0; i<numSites; i++)
		{
		if (site[i].isJC == NO)
			{
			site[i].branchSum = SumBranches (treeRoot[i]);
			totalBranchSum += site[i].branchSum;
			/* fprintf (stderr,"\nBranchSum [%d] = %6.4f", i, site[i].branchSum); */
			}
		}

	if (simulateJustSNPs == YES) /* Conditioned on the number of SNPs */
		numMutations = numSNPs;
	else		/* The number of mutations will be distributed as a Poisson with parameter totalBranchSum */
		numMutations = RandomPoisson (totalBranchSum, seed);
	
	/* If the number of ISM mutations is bigger than the number of ISM sites quit
		the program and warn the user about violation of ISM */
	if (numMutations > numISMsites)
		{
		fprintf (stderr, "\n\nERROR: The infinite sites model (ISM) has been violated. There are");
		fprintf (stderr, "\nmore mutatitons (%d) than sites under the ISM (%d).", numMutations, numISMsites);
		fprintf (stderr, "\nTry using a smaller mutation rate or effective population size");
		fprintf (stderr, "\n(i.e. smaller theta) or check the proportion of JC sites\n");
		exit (1);
		}

	mutations = 0;
	
	while (mutations < numMutations)
		{
		/* Choose a site according to its BranchSum */
		cumBranch = 0;
		uniform = RandomUniform(seed) * totalBranchSum;
		for (i=0; i<numSites; i++)
			{
			if (site[i].isJC == NO && site[i].done == NO)
				{
				cumBranch += site[i].branchSum;
				if (cumBranch >= uniform)
					{
					if (site[i].isSNP == NO)	
						{
						site[i].done = NO;
						SimulateISMForSite (treeRoot[i], i, numSites, mutationRate, seed);
						mutations++;
						site[i].isSNP = YES;
						totalBranchSum -= site[i].branchSum;
						
						#ifdef CHECK_MUT_DISTRIBUTION
							fprintf (fpSites,"%d\n", i+1);
						#endif
						}
					/* choose other site */
					break;
					}
				}
			}
		}
}



/********************************** SimulateISMForSite ***********************************/
/*	Simulates a mutation under an infinite sites model (ISM) for a given site. The branch
	where this mutation is place is chosen according to its length
*/

void SimulateISMForSite (TreeNode *p, int siteNum, int numSites, double m, long int *seed)
{
	
	static double	cumLength;
	double			uniform;

	uniform = RandomUniform(seed) * site[siteNum].branchSum;

	if (p != NULL)
		{
		if (p->anc == NULL)
			{
			matrix[pos(p->label,siteNum,numSites)] = 0;
			cumLength = 0;
			}
		else
			{
			cumLength += m*(p->anc->time - p->time);

			if (cumLength < uniform || site[siteNum].done == YES) /* => no change */
				{
				if (matrix[pos(p->anc->label,siteNum,numSites)] == 0)
					matrix[pos(p->label,siteNum,numSites)] = 0;
				else 
					matrix[pos(p->label,siteNum,numSites)] = 1;
				}
			else
				{
				if (matrix[pos(p->anc->label,siteNum,numSites)] == 0)
					matrix[pos(p->label,siteNum,numSites)] = 1;
				else 
					matrix[pos(p->label,siteNum,numSites)] = 0;
				
				site[siteNum].done = YES;
				site[siteNum].isSNP = YES;
				numMU++;
				
				}	
			}
		
		SimulateISMForSite (p->left,  siteNum, numSites, m, seed);
		SimulateISMForSite (p->right, siteNum, numSites, m, seed);	

		}
}
	



/************************************* SimulateJCForSite ***************************************/
/* Simulates the nucleotide substitution process for a given site under the Jukes-Cantor (1969)
	model for two alleles */

void SimulateJCForSite (TreeNode *p, int siteNum, int numSites, double m, long int *seed)
{
	double		probOfChange, uniform, branchLength;;
	
	if (p != NULL)
		{
		if (p->anc == NULL)
			matrix[pos(p->label,siteNum,numSites)] = 0;
		else
			{
			branchLength = m*(p->anc->time - p->time);
			uniform = RandomUniform(seed);
			probOfChange = 0.25 - 0.25 * exp (-branchLength);
			if (uniform >= probOfChange)
				{
				if (matrix[pos(p->anc->label,siteNum,numSites)] == 0)
					matrix[pos(p->label,siteNum,numSites)] = 0;
				else 
					matrix[pos(p->label,siteNum,numSites)] = 1;
				}
			else
				{
				if (matrix[pos(p->anc->label,siteNum,numSites)] == 0)
					matrix[pos(p->label,siteNum,numSites)] = 1;
				else 
					matrix[pos(p->label,siteNum,numSites)] = 0;
				numMU++;
				}	
			}
		
		SimulateJCForSite (p->left,  siteNum, numSites, m, seed);
		SimulateJCForSite (p->right, siteNum, numSites, m, seed);	

		}
}



/*************************** SumBranchLenghts **********************************/
/* Returns the sum of the branch lengths for the tree at a given site */
double SumBranches (TreeNode *p)
{
	static double sum;
	
	if (p != NULL)
		{
		if (p->anc == NULL)
			sum = 0;
		else
			sum += mutationRate * (p->anc->time - p->time);
		SumBranches (p->left);
		SumBranches (p->right);
		}

	return sum;
}






/********************* MakeGenotypes ********************/
/* creates diploid individuals by randomly mating haplotypes 
  and prints them to a file */

static void MakeGenotypes (long int *seed)
{
	int		*array;
	int		i, ind, gamete;
		
	individual =    (IndividualStr*)  calloc (numIndividuals, sizeof (IndividualStr)); 

	array =    (int*)  calloc (numGametes, sizeof (int)); 
	/* fill array from 0 to numGametes-1 */
	for (i=0; i<numGametes; i++)
		{
		array[i] = i;
		}				

	/* randomize array */
	RandomizeArray (array, 0, numGametes, seed);
	
	/* assign each consecutive pair to genotypes */
	for (ind=0, gamete=0; ind<numIndividuals; ind++)
		{
		individual[ind].haplotype1 = array[gamete++];
		individual[ind].haplotype2 = array[gamete++];
		}
	

	free (array);
}



/******************************** CalcIndividualG *********************************/
/* Calculates G, the number of potential locations in an active ancestral gamete 
   that a recombination event can take place. Keep track of the limits of the
   valid segment */

static int CalcIndividualG (int who, int *giBegin, int *giEnd, int numSites, TreeNode *nodes, int *activeGametes)
{
	int			j, k, g, position;
	TreeNode	*p, *q;
	
	g = 0;
	position = 0;
	p = nodes + pos(activeGametes[who],position,numSites);
	while (position < numSites  &&  (p->isAncestralSite == NO || (p->isAncestralSite == YES && MRCA[position] <= 1)))  
		{
		p++;
		position++;
		}
	if (position < numSites-1) /* from mark to end */
		{
		giBegin[who] = position;
		for (j=position+1; j<numSites; j++)
			{
			q = p;
			for (k=j; k<numSites; k++)
				{
				q++;  /* move potential breakpoint until find next ancestral stuff */
				if (q->isAncestralSite == YES && MRCA[k] > 1)
					{
					g++;
					break;
					}
				}
			p++;
			}
		}

	if (g > 0)
		giEnd[who] = giBegin[who] + g - 1;
	else
		giEnd[who] = giBegin[who] = -1;
	
	if (giEnd[who] < giBegin[who])
		{
		fprintf (stderr, "\n\nERROR: giEnd (%d) < giBegin (%d)", giEnd[who], giBegin[who]);
		exit(-1);
		}

	return g;
}





/******************************** IsValidBreakSite *********************************/
/* returns whether a site is potentially recombining: it has 
	   ancestral material non-MRCA before and after it */

static int 	IsValidBreakSite (int *activeGametes, int whichGamete, int whichSite, TreeNode *p)
{
	
	int		position, hasLeftAncestralMaterial, hasRightAncestralMaterial;  
	hasLeftAncestralMaterial = NO;
	hasRightAncestralMaterial = NO;	
	p = nodes + pos(activeGametes[whichGamete], 0, numSites);

	for (position=0; position < numSites; position++)
		{
			if (p->isAncestralSite == YES & MRCA[position] > 1)
				{
				if (position <= whichSite)
					{
					hasLeftAncestralMaterial = YES;
					/* fprintf (stderr,"l"); */
					}
				else
					{
					hasRightAncestralMaterial = YES;
					/* fprintf (stderr,"r"); */
					}
				}
			p++;
		}

	if (hasLeftAncestralMaterial == YES && hasRightAncestralMaterial == YES)
		return YES;
	else
		return NO;	
	
}



/*********************************** CountsForExpNumRec **********************************/
/* Checks whether a recombination event should be counted in the expected number of 
/* recombinations E(R). For E(R) count only events with breakpoints as X|X, 
/* where X is ancestral material (i.e, 1), but that did not find yet its MRCA   */                                                                   

static int CountsForExpNumRec (int *activeGametes, int whichGamete, int whichSite, TreeNode *nodes, int *MRCA)
{
	TreeNode	*p;
	p = nodes + pos(activeGametes[whichGamete], whichSite, numSites);
	if ((p->isAncestralSite == YES && MRCA[whichSite] > 1) && (((p+1)->isAncestralSite == YES) && MRCA[whichSite+1] > 1))
		return YES;
	else
		return NO;	
}



/************************* CountSNPs  ************************/
/* Count number of SNPs in a given data set */

int CountSNPs ()
{
	int		i, j;
	int		nSNPs, allele1, allele2;

	nSNPs = 0;
	for (j=0; j<numSites; j++)
		{
		allele1 = allele2 = NO;
		for (i=0; i<numGametes; i++)
			{
			if (matrix[pos(i,j,numSites)] == 0)
				allele1 = YES;
			else
				allele2 = YES;		
			}
		
		if (allele1 == YES && allele2 == YES)
			{
			nSNPs++;
			site[j].isSNP = YES;
			}
		}
	return nSNPs;	
}



/*************************** GammaHotspots **************************/
/*	Distances between hotspots are distributed as 
	gamma[interval, shape]. 
*/

int GammaHotspots (double lambda, double interval, long int *seed)
{

	int	i;
	int index, numberPoissonHotspots;
	int numThinnedHotspots;
	
	numberPoissonHotspots = PoissonHotspots (lambda, seed);
	
	/*
	fprintf (stderr, "\n\nBefore thinning: ");
	for (i=0; i<numberPoissonHotspots; i++)
		fprintf (stderr, "%d ", hotspot[i]);
	*/

	/* 	When there is no interference the model reduces to a Poisson 
		so we are done */
	if (interval<= 1)
		return numberPoissonHotspots;
	

	/*	If there is interference simulate
	   	gamma by thinning current hotspots */
	/*
		   origin
	- - - ----*-----------*----*--------* - - -

	*/
	
	/* Find ordinal for first hostpot (0->interval-1) */
	index = RandomUniformTo (interval, seed);
/*	fprintf (stderr, "\nIndex1 = %d\n\nAfter thinning: ", index);*/
	
	/* Keep only hotspots numbered as "0" */
	numThinnedHotspots = 0;
	for (i=0; i<numberPoissonHotspots; i++)
		{
		if (index == interval)
			index = 0;
		if (index == 0)
			{
			hotspot[numThinnedHotspots] = hotspot[i];
			numThinnedHotspots++;
			}
		index++;	
		}
	
	hotspot = (int*) realloc (hotspot, numThinnedHotspots*sizeof(int)); 

/*	for (i=0; i<numThinnedHotspots; i++)
		fprintf (stderr, "%d ", hotspot[i]);
	fprintf (stderr, "\n\n");
*/	
	
	return numThinnedHotspots;
}



/********************* PoissonHotspots ********************/
/* Generates a random number from a Poisson distibution with
   mean lambda. Uses the fact that the time between Poisson
   occurrences is exponentially distributed
*/

int PoissonHotspots (double lambda, long int *seed)
{
	int		i, poissonNumber, numberHotspots;
	double	sum;
	
	sum = 0;
	poissonNumber = -1;
	
	/* get poisson number and exponential position */
	while (sum <= 1.0)
		{
		sum += -(1.0/lambda) * log (RandomUniform(seed));
		poissonNumber++;
		if (sum < 1.0)
			{
			hotspot[poissonNumber] = (int) floor (sum * (numSites + 2*numSitesOutside - 1) + 0.5);
			if (poissonNumber > 0  &&  hotspot[poissonNumber] == hotspot[poissonNumber-1])
					hotspot[poissonNumber]++;
			}
		}
	
	numberHotspots = poissonNumber;
	
	/* make invalid ones to be like last valid */
		for (i=0; i<numberHotspots-1; i++)
			if (hotspot[i+1] >= numSites + 2*numSitesOutside)
				hotspot[i+1] = hotspot[i];
			

	/* Check whether the hotspots are valid */
	for (i=0; i<numberHotspots; i++)
		{
		if (hotspot[i] >= (numSites + 2*numSitesOutside)   ||  hotspot[i] < 0)
			{
			fprintf (stderr,"\n\nERROR: hotspot %d out of range (= %d)  numHotspots = %d  numSitesOutside = %d\n", 
						i+1, hotspot[i], numberHotspots, numSitesOutside);
			exit (-1);
			}
		}
	
	return numberHotspots;

}



/********************* UniformHotspots ********************/
/* 
	Generates a fixed number of hotspots locations from a 
  	uniform distribution. If a hotspot is overimposed upon another
  	it will be moved to the closest available location
*/

void UniformHotspots (int numberHotspots, long int *seed)
{
	int		i, j, isValid, moveLeft, sorted, pass, temp;	

	for (i=0; i<numberHotspots; i++)
		{
		/* get uniform location */
		hotspot[i] = (numSites - 1) * RandomUniform(seed);
		
		if (hotspot[i] < 0)
			{
			fprintf (stderr, "\nERROR: hotspot location is %d\n", hotspot[i]);
			exit(-1);
			}
		
		/* in case is needed, decide to move left or move right */
		if (hotspot[i] > ((numSites+ 2*numSitesOutside - 1)/2.0))
			moveLeft = YES;
		else
			moveLeft = NO;	

		isValid = NO;
		/* while this hotspot is occupied, move it one position at a time */
		while (isValid == NO)
			{
			isValid = YES;
			for (j=0; j<i; j++)
				{
				if (hotspot[i] == hotspot[j])
					{
					isValid = NO;
					if (moveLeft == NO)
						hotspot[i]++;
					else	
						hotspot[i]--;
					break;
					}
				}
			}
		}

	/* order hotspots from left to right */
	sorted = NO; 
	pass = 1; 
	while (sorted == NO)
		{
		sorted = YES;
		for (i=0; i < (numberHotspots-pass); i++)
	 		if (hotspot[i] > hotspot [i+1])
				{
				temp = hotspot[i];
				hotspot[i] = hotspot[i+1];
				hotspot[i+1] = temp;
				sorted = NO; 
				}
		pass++;	
		} 


	/* make invalid ones to be like last valid */
		for (i=0; i<numberHotspots-1; i++)
			if (hotspot[i+1] >= numSites + 2*numSitesOutside)
				hotspot[i+1] = hotspot[i];

	/* Check whether the hotspots are valid */
	for (i=0; i<numberHotspots; i++)
		{
		if (hotspot[i] >= (numSites + 2*numSitesOutside)   ||  hotspot[i] < 0)
			{
			fprintf (stderr,"\n\nERROR: hotspot %d out of range (= %d)  numHotspots = %d  numSitesOutside = %d\n", 
						i+1, hotspot[i], numberHotspots, numSitesOutside);
			exit (-1);
			}
		}

}



/************************ SimRecombinationPDF ***********************/
/* 
	Simulates a probability density function for recombination
	along the gene. The number of hotspots will be distributed 
	as a Poisson while their position is exponential. The position
	of the breakpoints follows a Normal (hotpost, imprecision) or  
	a Uniform(hotspotWidth). In the latter case we do not need to simulate.

	Added heterogeneity of hotspot recombination rate 
	(with gamma distribution)

*/

void SimRecombinationPDF (double *counts, long int *seed)
{
	int		i, k;
	int		breakpoint, before, after;
	double	gammaVariable;
	
	for (i=0; i<numHotspots; i++)
		{
		if (hotspotHeterogeneity == YES)
			gammaVariable = RandomGamma (alpha, seed) / alpha;
		else
			gammaVariable = 1;	

		if (hotspotUniform == YES) 
			{
			if (hotspotWidth == 1)
				counts[hotspot[i]] += gammaVariable * numPointsRecPDF;
			else
				{
				before = hotspotWidth/2.0;
				if (hotspotWidth%2 != 0)
					after = hotspotWidth/2.0 + 1;
				else
					after = hotspotWidth/2.0;	
				for (breakpoint = hotspot[i]-before; breakpoint < hotspot[i]+after; breakpoint++)
					 counts[breakpoint] += gammaVariable * numPointsRecPDF/(double)hotspotWidth;
				}
			}
		else
			{
			if (hotspotImprecision == 0)
				counts[hotspot[i]] += gammaVariable * numPointsRecPDF;
			else
				{
				for (k=0; k<numPointsRecPDF; k++)
					{
					breakpoint = (int) floor (RandomNormal (hotspot[i], hotspotImprecision, seed) + 0.5);
					if (breakpoint < 0 || breakpoint >= numSites+2.0*numSitesOutside)
						continue;
					counts[breakpoint] += gammaVariable;
					}
				}
			}
		}
}


/******************** ReadParametersFromCommandLine **************************/
static void ReadParametersFromCommandLine (int argc,char **argv)
{
	int		i, j;
	char	flag; 
 	float	argument;

	for (i=1; i<argc; i++)
	{
		argv[i]++;
		flag=*argv[i];
		argv[i]++;
		argument = -9999;
		
	/* A B C D E F G H I J K L M N O P Q R S T U V W X Y Z * 1 ? */
	    switch (toupper(flag))
			{
			case '#':
				argument = atof(argv[i]);
				userSeed = (int) argument;
				if (userSeed < 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad user seed for random number (%d)\n\n", userSeed);
					PrintUsage();
					}
			break;
			case 'S':
				argument = atof(argv[i]);
				numGametes = (int) argument;
				if (numGametes < 1)
				 	{
					fprintf (stderr, "PARAMETER ERROR: Bad sample size (%d)\n\n", numGametes);
					PrintUsage();
					}
				/* Check that the number of haplotypes is even */
				if (numGametes&2 != 0)
					{
					fprintf (stderr,"\n\nWARNING: number of gametes was odd (%d), so it has been",  numGametes++);
					fprintf (stderr,"\naugmented to %d to make an even number of individuals\n\n", numGametes);
					}
				numIndividuals = numGametes / 2.0;
			break;
			case 'L':
				argument = atof(argv[i]);
				numSites = (int) argument;
				if (numSites<1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad sequence length (%d)\n\n", numSites);
					PrintUsage();
					}
			break;
			case 'E':
				argument = atof(argv[i]);
				N = (int) argument;
				if (N < 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad effective population size (%d)\n\n", N);
					PrintUsage();
					}
			break;		
			case 'R':
	        	recombinationRate = atof(argv[i]);
				if (recombinationRate < 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad homogeneous recombination rate (%f)\n\n",recombinationRate );
					PrintUsage();
					}
			break;
			case 'K':
	        	blockRecombinationRate = atof(argv[i]);
				if (blockRecombinationRate < 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad block recombination rate (%f)\n\n", blockRecombinationRate);
					PrintUsage();
					}
				if (blockRecombinationRate == 0) 
        			doBlockRecombination = NO;
        		else
        			doBlockRecombination = YES;
			break;
			case 'T':
	        	alpha = atof(argv[i]);
				if (alpha < 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad gamma shape (alpha) for hotspot recombination heterogeneity (%f)\n\n", alpha);
					PrintUsage();
					}
				hotspotHeterogeneity = YES;
			break;
			case 'V':
	        	hotspotImprecision = atof(argv[i]);
				if (hotspotImprecision < 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad hotspot imprecision (%f)\n\n", hotspotImprecision);
					PrintUsage();
					}
				if (hotspotWidth > 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: You cannot specify hotspot imprecision (Normal) and width (Uniform) simultaneosuly \n\n");
					PrintUsage();
					}
			break;
			case 'Z':
				argument = atof(argv[i]);
				interferenceInterval = (int) argument;
				if (interferenceInterval <1)
					{
					fprintf (stderr, "PARAMETER ERROR: Bad interference interval (%d)\n\n", interferenceInterval);
					PrintUsage();
					}
				hotspotInterference = YES;
			break;
			case 'M':
				argument = atof(argv[i]);
				hotspotWidth = (int) argument;
				hotspotUniform = YES;
				if (hotspotWidth < 1)
					{
					fprintf (stderr, "PARAMETER ERROR: Bad hotspot width (%d < 1)\n\n", hotspotWidth);
					PrintUsage();
					}
				if (hotspotImprecision > 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: You cannot specify hotspot imprecision (Normal) and width (Uniform) simultaneosuly \n\n");
					PrintUsage();
					}
			break;
			case 'B':
	        	growthRate = atof(argv[i]);
				if (growthRate < 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad growth rate (%f)\n\n", growthRate);
					PrintUsage();
					}
				if (growthRate != 0) 
        			{
        			doExponential = YES;
		        	if (doDemographics == YES || doLogistic == YES)
						{
						fprintf (stderr, "PARAMETER ERROR: Cannot have both exponential (-b) and  demographics (-d -c)\n\n");
						exit (1);
						}
					}
			break;
			case 'C':
				argument = atof(argv[i]);
	        	carryingCapacity = (int) argument;
				if (carryingCapacity <= 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad carrying capacity for logistic model (%f) \n\n", carryingCapacity);
					PrintUsage();
					}
				if (growthRate != 0) 
        			{
        			doLogistic = YES;
		        	if (doDemographics == YES || doExponential == YES)
						{
						fprintf (stderr, "PARAMETER ERROR: Cannot have both logistic (-c) and other demographics (-b -d)\n\n");
						exit (1);
						}
					}
				if (growthRate < 0 && carryingCapacity < N)
					{
					fprintf (stderr, "PARAMETER ERROR: Cannot have negative growth rate and a carrying capacity smaller than the present effective size\n\n");
					exit (1);
					
					}
			break;
			case 'D':
				doDemographics = YES;
		        if (doExponential == YES || doLogistic == YES)
					{
					fprintf (stderr, "PARAMETER ERROR: Cannot have both demographics periods (-d) and other demographics (-b -c)\n\n");
					exit (1);
					}
	        	argument = atof(argv[i]);
				numPeriods = (int) argument;
				if (numPeriods <= 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad number of periods (%d)\n\n", numPeriods);
					PrintUsage();
					}
				Nbegin = 	(int *) calloc(numPeriods+1, sizeof(int));
				Nend =		(int *) calloc(numPeriods+1, sizeof(int));
				cumDuration =	(int *) calloc(numPeriods+1, sizeof(int));
				periodGrowth =	(double *) calloc(numPeriods+1, sizeof(double));
				if (Nbegin == NULL || Nend == NULL || cumDuration == NULL)
					{
					fprintf (stderr, "PARAMETER ERROR: Could not allocate demographic vectors (*d)\n", numPeriods * sizeof(int));
					exit (1);
					}
				for (j=1; j<=numPeriods; j++)
					{
					argument = atof(argv[++i]);
	        		Nbegin[j] = (int) argument;
					argument = atof(argv[++i]);
        			Nend[j] = (int) argument;
					argument = atof(argv[++i]);
	        		cumDuration[j] = (int) argument + cumDuration[j-1];
					}
			break;
			case 'U':
	        	mutationRate = atof(argv[i]);
				if (mutationRate < 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad mutation rate (%f)\n\n", mutationRate);
					PrintUsage();
					}
			break;
			case 'W':
	        	JCrelMutationRate = atof(argv[i]);
				if (JCrelMutationRate < 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad relative JC mutation rate (%f)\n\n", JCrelMutationRate);
					PrintUsage();
					}
			break;
			case 'X':
				argument = atof(argv[i]);
				numSNPs = (int) argument;
				if (numSNPs <1)
					{
					fprintf (stderr, "PARAMETER ERROR: Bad number of SNPs (%d)\n\n", numSNPs);
					PrintUsage();
					}
				simulateJustSNPs = YES;
				if (propJCsites > 0)
					{
					fprintf (stderr, "PARAMETER ERROR: You cannot specify a number of SNPs if there is any JC site. Set the proportion of JC to zero\n\n");
					PrintUsage();
					}
			break;
			case 'N':
				argument = atof(argv[i]);
				numDataSets = (int) argument;
				if (numDataSets <1)
					{
					fprintf (stderr, "PARAMETER ERROR: Bad number of replicates (%d)\n\n", numDataSets);
					PrintUsage();
					}
			break;
			case 'Y':
				argument = atof(argv[i]);
				noisy = (int) argument;
				if (noisy < 0)
					{
					fprintf (stderr, "PARAMETER ERROR: Bad noisy value (%d)\n\n", noisy);
					PrintUsage();
					}
			break;
			case 'G':
				argument = atof(argv[i]);
				numNodes = (int) argument;
				if (numNodes<1)
					{
					fprintf (stderr, "PARAMETER ERROR: Bad number of nodes (%d)\n\n", numNodes);
					PrintUsage();
					}
			break;
			case 'P':
	        	propJCsites = atof(argv[i]);
				if (propJCsites < 0 || propJCsites > 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad proportion of fast sites (%f). It has to be between 0 and 1\n\n", propJCsites);
					PrintUsage();
					}
			break;
			case 'H':
				expNumPoissonHotspots = atof(argv[i]);
				if (expNumPoissonHotspots < 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad number of expected hotspots (%6.4f)\n\n", expNumPoissonHotspots);
					PrintUsage();
					}
				if (fixedNumHotspots > 0)
					{
					fprintf (stderr, "PARAMETER ERROR: Do not specify at the same time an expected and a fixed number of hotspots\n\n");
					PrintUsage();
					}	
			break;
			case 'Q':
				fixedNumHotspots = atof(argv[i]);
				if (fixedNumHotspots<1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad fixed number of hotspots (%6.4f)\n\n", expNumPoissonHotspots);
					PrintUsage();
					}
				if (expNumPoissonHotspots > 0)
					{
					fprintf (stderr, "PARAMETER ERROR: Do not specify at the same time an expected and a fixed number of hotspots\n\n");
					PrintUsage();
					}	
			break;		
			case '1':
				simData = NO;
			break;
			case 'A':
			 	doPrintSequences = YES;
			break;
			case 'F':
				doPrintGenotypes = YES;
			break;
			case 'I':
				doPrintTrees = YES;
			break;
			case 'J':
				doPrintTimes = YES;
			break;
			case 'O':
			 	doPrintMutationModel = YES;
			break;
			case '?':
				PrintUsage();
			break;
			default :
				fprintf (stderr, "PARAMETER ERROR: Incorrect parameter: %c\n\n", flag);
				PrintUsage();
			break;
		}
	} 
}



/***************************** ReadParametersFromFile *******************************/
/* Reads parameter values from the parameter file  */

void ReadParametersFromFile ()
{
	int		j;
	char 	ch;
	float	argument;

	
	if(feof(stdin))
		{
		fprintf (stderr, "FILE ERROR: Unable to read parameters from stdin\n");
		exit(0);
		}

	ch=fgetc(stdin);
	while(isspace(ch))
		ch=fgetc(stdin);
	while(ch=='[')
		{
		ReadUntil(stdin, ']', "closing bracket");
		ch=fgetc(stdin);
		while(isspace(ch))
			ch=fgetc(stdin);
		}
	
	/* A B C D E F G H I J K L M N O P Q R S T U V W X Y Z * 1 ? */
	while(!feof(stdin))
		{
		argument = 0;
		ch=toupper(ch);
		switch (ch) 
			{
			case '#':
				if (fscanf(stdin, "%f", &argument) !=1 || argument < 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad seed (%d)\n\n", (int) argument);
					PrintUsage();
					}
				userSeed = (int) argument;
			break;
			case 'S':
				if (fscanf(stdin, "%f", &argument) !=1 || argument < 1)
				 	{
					fprintf (stderr, "PARAMETER ERROR: Bad sample size (%d)\n\n", (int) argument);
					PrintUsage();
					}
				numGametes = (int) argument;
				/* Check that the number of haplotypes is even */
				if (numGametes&2 != 0)
					{
					fprintf (stderr,"\n\nWARNING: number of gametes was odd (%d), so it has been",  numGametes++);
					fprintf (stderr,"\naugmented to %d to make an even number of individuals\n\n", numGametes);
					}
				numIndividuals = numGametes / 2.0;
			break;
			case 'L':
				if (fscanf(stdin, "%f", &argument) !=1 || argument < 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad sequence length (%d)\n\n", (int) argument);
					PrintUsage();
					}
				numSites = (int) argument;
			break;
			case 'E':
				if (fscanf(stdin, "%f", &argument) !=1 || argument < 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad effective population size (%d)\n\n", (int) argument);
					PrintUsage();
					}
				N = (int) argument;
			break;		
			case 'R':
				if (fscanf(stdin, "%lf", &recombinationRate) !=1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad homogeneous recombination rate (%f)\n\n", recombinationRate);
					PrintUsage();
					}
			break;
			case 'K':
				if (fscanf(stdin, "%lf", &blockRecombinationRate) !=1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad block recombination rate (%f)\n\n", blockRecombinationRate);
					PrintUsage();
					}
				if (blockRecombinationRate == 0) 
        			doBlockRecombination = NO;
        		else
        			doBlockRecombination = YES;
			break;
			case 'B':
				if (fscanf(stdin, "%lf", &growthRate) !=1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad growth rate (%f)\n\n", growthRate);
					PrintUsage();
					}
				if (growthRate != 0) 
	    			{
	    			doExponential = YES;
		        	if (doDemographics == YES || doLogistic == YES)
						{
						fprintf (stderr, "PARAMETER ERROR: Cannot have both exponential (-b) and other demographics(-d -c)\n\n");
						exit (1);
						}
					}
			break;
			case 'C':
				if (fscanf(stdin, "%f", &argument) !=1 || argument < 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad logistic carrying capacity (%f)\n\n", argument);
					PrintUsage();
					}
				carryingCapacity = (int) argument;
				if (growthRate != 0) 
	    			{
	    			doLogistic = YES;
		        	if (doDemographics == YES || doExponential == YES)
						{
						fprintf (stderr, "PARAMETER ERROR: Cannot have both logistic (-c) and other demographics(-d -b)\n\n");
						exit (1);
						}
					}
			break;
			case 'D':
				doDemographics = YES;
	        	if (doExponential == YES || doLogistic == YES)
					{
					fprintf (stderr, "PARAMETER ERROR: Cannot have both demographics periods (-d) and other demographics (-b -c)\n\n");
					exit (1);
					}
				if (fscanf(stdin, "%f", &argument) !=1 || argument < 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad number of periods (%d)\n\n", (int) argument);
					PrintUsage();
					}
				numPeriods = (int) argument;
				Nbegin =	(int *) calloc(numPeriods+1, sizeof(int));
				Nend = 		(int *) calloc(numPeriods+1, sizeof(int));
				cumDuration =	(int *) calloc(numPeriods+1, sizeof(int));
				periodGrowth =	(double *) calloc(numPeriods+1, sizeof(double));
				if (Nbegin == NULL || Nend == NULL || cumDuration == NULL)
					{
					fprintf (stderr, "PARAMETER ERROR: Could not allocate demographic vectors (%d)\n", numPeriods * sizeof(int));
					exit (1);
					}
				for (j=1; j<=numPeriods; j++)
					{
					fscanf(stdin, "%f", &argument);
					Nbegin[j] = (int) argument;
					fscanf(stdin, "%f", &argument);
					Nend[j] = (int) argument;
					fscanf(stdin, "%f", &argument);
					cumDuration[j] = (int) argument + cumDuration[j-1];
					}
			break;
			case 'V':
				if (fscanf(stdin, "%lf", &hotspotImprecision) !=1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad hotspot imprecision (%f)\n\n", hotspotImprecision);
					PrintUsage();
					}
				if (hotspotWidth > 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: You cannot specify hotspot imprecision (Normal) and width (Uniform) simultaneosuly \n\n");
					PrintUsage();
					}
			break;
			case 'Z':
				if (fscanf(stdin, "%f", &argument) !=1 || argument < 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad interference interval (%d)\n\n", (int) argument);
					PrintUsage();
					}
				interferenceInterval = (int) argument;
				hotspotInterference = YES;
			break;
			case 'M':
				if (fscanf(stdin, "%f", &argument) !=1 || argument < 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad hotspot width (%d<1)\n\n", (int) argument);
					PrintUsage();
					}
				hotspotWidth = (int) argument;
				hotspotUniform = YES;
				if (hotspotImprecision > 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: You cannot specify hotspot imprecision (Normal) and width (Uniform) simultaneosuly \n\n");
					PrintUsage();
					}
			break;
			case 'T':
				if (fscanf(stdin, "%lf", &alpha)!=1 || alpha < 0) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad alpha shape for hotspot recombination heterogeneity (%f)\n\n", alpha);
					PrintUsage();
					}
				hotspotHeterogeneity = YES;
			break;
			case 'U':
				if (fscanf(stdin, "%lf", &mutationRate)!=1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad mutation rate (%f)\n\n", mutationRate);
					PrintUsage();
					}
			break;
			case 'X':
				if (fscanf(stdin, "%f", &argument) !=1 || argument < 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad number of SNPs (%d)\n\n", (int) argument);
					PrintUsage();
					}
				numSNPs = (int) argument;
				simulateJustSNPs = YES;
				if (propJCsites > 0)
					{
					fprintf (stderr, "PARAMETER ERROR: You cannot specify a number of SNPs if there is any JC site. Set the proportion of JC to zero\n\n");
					PrintUsage();
					}
			break;
			case 'W':
				if (fscanf(stdin, "%lf", &JCrelMutationRate)!=1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad relative JC mutation rate (%f)\n\n", JCrelMutationRate);
					PrintUsage();
					}
			break;
			case 'N':
				if (fscanf(stdin, "%f", &argument) !=1 || argument < 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad number of replicates (%d)\n\n", (int) argument);
					PrintUsage();
					}
				numDataSets = (int) argument;
			break;
			case 'Y':
				if (fscanf(stdin, "%f", &argument) !=1 || argument < 0)
					{
					fprintf (stderr, "PARAMETER ERROR: Bad noisy value (%d)\n\n", (int) argument);
					PrintUsage();
					}
				noisy = (int) argument;
			break;
			case 'G':
				if (fscanf(stdin, "%f", &argument) !=1 || argument < 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad number of nodes value (%d)\n\n", (int) argument);
					PrintUsage();
					}
				numNodes = (int) argument;
			break;
			case 'P':
				if (fscanf(stdin, "%lf", &propJCsites) !=1 || propJCsites < 0 || propJCsites > 1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad proportion of fast JC (%f)\n\n", propJCsites);
					PrintUsage();
					}
			break;
			case 'H':
				if (fscanf(stdin, "%lf", &expNumPoissonHotspots) !=1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad number of expected hotspots (%f)\n\n", expNumPoissonHotspots );
					PrintUsage();
					}
				if (fixedNumHotspots > 0)
					{
					fprintf (stderr, "PARAMETER ERROR: Do not specify both an expected and a fixed number of hotspots\n\n");
					PrintUsage();
					}	
	        break;
			case 'Q':
				if (fscanf(stdin, "%lf", &fixedNumHotspots)!=1) 
					{
					fprintf (stderr, "PARAMETER ERROR: Bad fixed number of hotspots (%f)\n\n", fixedNumHotspots );
					PrintUsage();
					}
				if (expNumPoissonHotspots > 0)
					{
					fprintf (stderr, "PARAMETER ERROR: Do not specify at the same time an expected and a fixed number of hotspots\n\n");
					PrintUsage();
					}	
			break;
			case '1':
				simData = NO;
			break;
			case 'A':
				doPrintSequences = YES;
			break;
			case 'F':
				doPrintGenotypes = YES;
			break;
			case 'I':
				doPrintTrees = YES;
			break;
			case 'J':
				doPrintTimes = YES;
			break;
			case 'O':
				doPrintMutationModel = YES;
			break;
			case '?':
				PrintUsage();
			break;
			default :
				fprintf (stderr, "PARAMETER ERROR: Incorrect parameter: %c\n\n", ch);
				PrintUsage();
			break;
		}
		ch=fgetc(stdin);
		while(isspace(ch) && !feof(stdin))
			ch=fgetc(stdin);
		while(ch=='[')
			{
			ReadUntil(stdin, ']', "closing bracket");
			ch=fgetc(stdin);
			while(isspace(ch))
				ch=fgetc(stdin);
			}
		}

}




/********************* PrepareFiles **********************/
/* Open files to output results */

void PrepareFiles()
{

	char File[MAX_NAME];
	strcpy(dirname, "Results");

	mkdir(dirname,S_IRWXU);

	if (doPrintSNPs == YES)
		{
		#ifdef macintosh
			sprintf(File,":%s:%s", dirname, SNPfile);
		#else
			sprintf(File,"%s/%s", dirname, SNPfile);
		#endif
		if ((fpSNPs = fopen(File, "w")) == NULL) 
			{
			fprintf (stderr, "Can't open \"%s\"\n", File);
			exit(-1);
			}

		#ifdef macintosh
			sprintf(File,":%s:%s", dirname, haplotypeFile);
		#else
			sprintf(File,"%s/%s", dirname, haplotypeFile);
		#endif
		if ((fpHaplotypes = fopen(File, "w")) == NULL) 
			{
			fprintf (stderr, "Can't open \"%s\"\n", File);
			exit(-1);
			}
		}
	if (doPrintSequences == YES)	
		{
		#ifdef macintosh
			sprintf(File,":%s:%s", dirname, sequenceFile);
		#else
			sprintf(File,"%s/%s", dirname, sequenceFile);
		#endif
		if ((fpSequences = fopen(File, "w")) == NULL) 
			{
			fprintf (stderr, "Can't open \"%s\"\n", File);
			exit(-1);
			}
		}
	if (doPrintGenotypes == YES)	
		{
		#ifdef macintosh
			sprintf(File,":%s:%s", dirname, genotypeFile);
		#else
			sprintf(File,"%s/%s", dirname, genotypeFile);
		#endif
		if ((fpGenotypes = fopen(File, "w")) == NULL) 
			{
			fprintf (stderr, "Can't open \"%s\"\n", File);
			exit(-1);
			}
		}
	if (doPrintTrees == YES)	
		{
		#ifdef macintosh
			sprintf(File,":%s:%s", dirname, treeFile);
		#else
			sprintf(File,"%s/%s", dirname, treeFile);
		#endif
		if ((fpTrees = fopen(File, "w")) == NULL) 
			{
			fprintf (stderr, "Can't open \"%s\"\n", File);
			exit(-1);
			}
		}
	if (doPrintTimes == YES)
		{
		#ifdef macintosh
			sprintf(File,":%s:%s", dirname, timesFile);
		#else
			sprintf(File,"%s/%s", dirname, timesFile);
		#endif
		if ((fpTimes = fopen(File, "w")) == NULL) 
			{
			fprintf (stderr, "Can't open \"%s\"\n", File);
			exit(-1);
			}
		}
	if (doPrintMutationModel == YES)
		{
		#ifdef macintosh
			sprintf(File,":%s:%s", dirname, mutationsFile);
		#else
			sprintf(File,"%s/%s", dirname, mutationsFile);
		#endif
		if ((fpMutations = fopen(File, "w")) == NULL) 
			{
			fprintf (stderr, "Can't open \"%s\"\n", File);
			exit(-1);
			}
		}

}


/******************* WriteTree ****************/
/* Writes a given tree from PrintTrees */

void WriteTree (TreeNode *p)
{		
	if (p != NULL)
		{
		if (p->left == NULL && p->right == NULL)
			{
			/*fprintf (fpTrees, "seq%05d:%5.4f", Label(p),(p->anc->time-p->time)*mutationRate);*/
			fprintf (fpTrees, "seq%05d:%5.4f", Index (p),(p->anc->time-p->time)*mutationRate);
			}
		else
			{
			fprintf (fpTrees, "(");
			WriteTree (p->left);
			fprintf (fpTrees, ",");
			WriteTree (p->right);	
			if (p->anc !=NULL)
				fprintf (fpTrees, "):%5.4f",(p->anc->time-p->time)*mutationRate);
			}
		}
}





/********************** ListTimes ************************/
/*	Writes a given tree description from ListTimes 
	It does not list superfluous nodes
*/

void ListTimes (int position)
{
	TreeNode	*p;
	int			i = 0;
	
	do
		{
		p = nodes + pos(i,position,numSites);

		if (p->anc != NULL && p->left != NULL & p->right != NULL)
			fprintf (fpTimes, "%8s   %4d   %4d  (%4d %4d %4d) |   %10.2lf      %10.2lf       %10.4lf\n", 
				"internal", Label(p), Index(p), Index(p->left), Index(p->right), Index(p->anc), p->time, p->anc->time - p->time, (p->anc->time - p->time)*mutationRate);
		else if (p->anc != NULL && p->left == NULL & p->right == NULL)
			fprintf (fpTimes, "%8s   %4d   %4d  (%4d %4d %4d) |   %10.2lf      %10.2lf       %10.4lf\n", 
				"tip", Label(p), Index(p), Index(p->left), Index(p->right), Index(p->anc), p->time, p->anc->time - p->time, (p->anc->time - p->time)*mutationRate);
		else if (p->anc == NULL && p->left != NULL && p->right != NULL)
			fprintf (fpTimes, "%8s   %4d   %4d  (%4d %4d %4d) |   %10.2lf      %10.2lf       %10.4lf\n", 
				"root", Label(p), Index(p), Index(p->left), Index(p->right), Index(p->anc), p->time, 0.0, 0.0);
		else
			fprintf (fpTimes,"");
			
		i++;
		
		if (i>2000) /* just in case the tree is weird */
			{
			fprintf (stderr, "\nProblems when listing times in the tree");
			exit(-1);
			}

		} while    (p->anc  != NULL || p->left == NULL);
}







/********************* PrintTimes **********************/
/*	Prints to timesfile a detailed description of
	the tree: nodes, times, branch lengths. 
*/

void PrintTimes(int replicate)
{
	int		j;
	
	if (numRE == 0)
		{
		fprintf (fpTimes, "\n----------------- Nodes -----------------");
		fprintf (fpTimes, "\n   class  label  index  (left right anc) |         time     time length    branch lenght");
		fprintf (fpTimes, "\n-----------------------------------------------------------------------------------------\n");
		ListTimes (0);
		}
	else
		{
		for (j=0; j<numSites; j++)
			{
			fprintf (fpTimes, "\n\nDataset %d  Site %d", replicate+1, j+1);
			fprintf (fpTimes, "\n-------------- Nodes -------------");
			fprintf (fpTimes, "\n   class  index  (left right anc) |         time     time length    branch lenght");
			fprintf (fpTimes, "\n----------------------------------------------------------------------------------\n");
			ListTimes (j);
			}
		}
}



/**************** Print Trees ***************/
/*  Print trees to treefile in Newick format */

void PrintTrees (int replicate)
{
	int		j;

	replicate = replicate;
	
	if (numRE == 0)
		{
		/*fprintf (fpTrees,"Tree.%05d = ", replicate+1);*/
		WriteTree (treeRoot[0]);
		fprintf (fpTrees,");\n");
		}
	else
		{
		for (j=0; j<numSites; j++)
			{
			/*fprintf (fpTrees, "Tree.%05d.%05d = ", replicate+1, j+1);*/
			WriteTree (treeRoot[j]);
			fprintf (fpTrees, ");\n");
			}
		}

	fprintf (fpTrees,"\n");
}



/************************ PrintSNPs ***********************/
/* prints only SNPs in a specific format for the simulations */

static void PrintSNPs ()
{
	int		i, j, ind;
		
	fprintf (fpSNPs, "%d\n", numSNPs);
	for (i=0; i<numSites; i++)
		if (site[i].isSNP == YES)
			fprintf (fpSNPs, "%2d ", i+1);
		
	fprintf (fpSNPs, "\n");

	for (ind=0; ind<numIndividuals; ind++)
		{
		for (j=0; j<numSites-1; j++)
			if (site[j].isSNP == YES)
				fprintf (fpSNPs, "%d%d ", matrix[pos(individual[ind].haplotype1,j, numSites)], 
								matrix[pos(individual[ind].haplotype2, j, numSites)]);
		if (site[j].isSNP == YES)
			fprintf (fpSNPs, "%d%d", matrix[pos(individual[ind].haplotype1,j, numSites)], 
							matrix[pos(individual[ind].haplotype2, j, numSites)]);
		fprintf (fpSNPs,"\n");
		}	


}	

/***************************** PrintHaplotypes *******************************/
/* Prints gametes with SNP sites to a file */

static void PrintHaplotypes ()
{
	int		 i, j;

	fprintf (fpHaplotypes,"%d %d\n",numGametes, numSites);
	for (i=0; i<numGametes; i++)
		{
		fprintf (fpHaplotypes,"seq%05d   ", i+1);
		for (j=0; j<numSites; j++)
			if (site[j].isSNP == YES)
				fprintf (fpHaplotypes, "%d", matrix[pos(i,j,numSites)]);
		fprintf (fpHaplotypes,"\n");
		}	
}




/********************* PrintGenotypes ********************/
/* Prints genotypes at all sites (variable + invariable) to a file */ 

static void PrintGenotypes ()
{
	int		j, ind;
	
	fprintf (fpGenotypes,"%d %d\n",numIndividuals, numSites);

	for (ind=0; ind<numIndividuals; ind++)
		{
		fprintf (fpGenotypes,"ind%d(%d,%d)\t\t", ind+1, individual[ind].haplotype1+1, individual[ind].haplotype2+1);

		for (j=0; j<numSites; j++)
			fprintf (fpGenotypes, " %d%d", matrix[pos(individual[ind].haplotype1,j, numSites)], 
						matrix[pos(individual[ind].haplotype2, j, numSites)]);
		fprintf (fpGenotypes,"\n");
		}	
			
}



/***************************** PrintSequences *******************************/
/* Prints gametes with all sites (variable + invariable) to a file */

static void PrintSequences ()
{
	int		 i, j;

	fprintf (fpSequences,"%d %d\n",numGametes, numSites);
	for (i=0; i<numGametes; i++)
		{
		fprintf (fpSequences,"seq%05d   ", i+1);
		for (j=0; j<numSites; j++)
			fprintf (fpSequences, "%d", matrix[pos(i,j,numSites)]);
		fprintf (fpSequences,"\n");
		}	
}
		

/************************************* PrintMutationModel ***************************************/
/* Prints to a file whether a site was evolved under the ISM or JC model */

void PrintMutationModel (int replicate)
{
	int	i;
	
	fprintf (fpMutations, "\n\nDataset %d\n", replicate+1);
	for (i=0; i<numSites; i++)
		{
		if (site[i].isJC == NO)
			fprintf (fpMutations, "Site %3d =\tISM\n", i+1);
		else
			fprintf (fpMutations, "Site %3d =\tJC\n", i+1);
		}	
}



/***************************** PrintRunSettings *******************************/
/* Prints a summary of run settings */

static void	PrintRunSettings (FILE *filep, long int seed)
{
	int		i;

	fprintf (filep, "\n\nRun settings\n------------");
	fprintf (filep, "\n[Assumptions in brackets]\n");
	fprintf (filep, "\n Seed                                         =   %-3d",  seed);
	fprintf (filep, "\n Number replicate data sets                   =   %-3d",  numDataSets);
	fprintf (filep, "\n Number of gametes                            =   %-3d",  numGametes);
	fprintf (filep, "\n Number of sites                              =   %-3d",  numSites);

	fprintf (filep, "\n\nDemographics");
	fprintf (filep, "\n Effective population size                    =   %-3d",  N);
	if (doExponential == YES)
		fprintf (filep, "\n Exponential growth rate                      =   %+2.1e", growthRate);
	else if (doLogistic == YES)
		{
		fprintf (filep, "\n Logistic growth rate                      =   %+2.1e", growthRate);
		fprintf (filep, "\n Carrying capacity                         =   %-3d", carryingCapacity);
		}
	else if (doDemographics == YES)
		{
		fprintf (filep, "\n Period   Nbegin     Nend    Duration    Growth rate");
		for (i=1; i<=numPeriods; i++)
			fprintf (filep, "\n   %d    %7d   %7d     %7d       %+2.1e", 
			i, Nbegin[i], Nend[i], cumDuration[i]-cumDuration[i-1], periodGrowth[i]);	
		}
		
	fprintf (filep, "\n\nCoalescent");
	fprintf (filep, "\n Mean number of coalescence events            =   %3.2f", meanNumCA);
	fprintf (filep, "\n Mean time to the last MRCA (# generations)   =   %3.2f", meanTMRCA);
	fprintf (filep, "\n Expected time to the MRCA [N=con, no rec]    =   %3.2f", expTMRCA);
	if (numDataSets > 1)
		{
		fprintf (filep, "\n Variance time MRCA                           =   %3.2f", varTMRCA);
		fprintf (filep, "\n Expected variance time MRCA [N=con, no rec]  =   %3.2f", expVarTMRCA);
		}
	fprintf (filep, "\n\nTotal recombination");
	fprintf (filep, "\n Population recombination parameter (R=Rb+Rh) =   %3.2f", rho + tau);
	fprintf (filep, "\n Global recombination rate / site gen (r_g)   =   %2.1e", globalRecombinationRate);
	if (doBlockRecombination == YES)
		{
		fprintf (filep, "\n Mean overall recombination rate (PDF C01)    =   %3.2f", meanC01);
		fprintf (filep, "\n Variance overall recombination rate          =   %3.2f", varC01);
		}
	fprintf (filep, "\n Mean number recombination events             =   %3.2f", meanNumRE);
	if (numDataSets > 1)
		fprintf (filep, "\n Variance number of recombination events      =   %3.2f", varNumRE);
	fprintf (filep, "\n Countable recombination events               =   %3.2f", meanNumREtc);
	fprintf (filep, "\n Expected number of countable rec events      =   %3.2f", expNumRE);
	fprintf (filep, "\n    [N=con, hotspot imprecision > 0]");	
	if (numDataSets > 1)
	fprintf (filep, "\n Variance countable rec events                =   %3.2f", varNumREtc);
	fprintf (filep, "\n Proportion of reps without recombination     =   %3.2f", zeroRec/(double)numDataSets);

	fprintf (filep, "\n\nBackground recombination");
	fprintf (filep, "\n Background pop rec parameter (Rb=4Nr_hL)     =   %3.2f", rho);
	fprintf (filep, "\n Recombination rate / site gen (r_h)          =   %2.1e", recombinationRate);

	fprintf (filep, "\n\nHotspot recombination");
	fprintf (filep, "\n Hotspot pop rec parameter (Rh=4Nr_bE(X))     =   %3.2f", tau);
	fprintf (filep, "\n Hotspot recombination rate / site gen(r_b)   =   %2.1e", blockRecombinationRate);
	if (fixedNumHotspots > 0)
		fprintf (filep, "\n Fixed number hotspots                        =   %3.2f",  fixedNumHotspots);
	else
		fprintf (filep, "\n Expected number hotspots                     =   %3.2f",  expNumHotspots);
	fprintf (filep, "\n Mean number observed hotspots                =   %3.2f", meanNumHotspots);
	if (numDataSets > 1)
		fprintf (filep, "\n Variance number observed hotspots            =   %3.2f", varNumHotspots);
	if (hotspotUniform == NO)
		fprintf (filep, "\n Hotspot imprecision                          =   %2.1e", hotspotImprecision);
	else
		fprintf (filep, "\n Hotspot width                                =   %-3d", hotspotWidth);
	if (hotspotHeterogeneity == YES)
		fprintf (filep, "\n Hotspot heterogeneity: gamma shape           =   %2.1e",  alpha);
	if (hotspotInterference == YES)
		fprintf (filep, "\n Hotspot interference interval                =   %-3d", interferenceInterval);

	if (simData == YES)
		{
		fprintf (filep, "\n\nMutations and SNPs");
		fprintf (filep, "\n Mutation rate                                =   %2.1e", mutationRate);
		fprintf (filep, "\n Population mutation parameter                =   %3.2f", theta);
		fprintf (filep, "\n   [Theta=4NuL for ISM]");
		fprintf (filep, "\n Mean number of mutation events               =   %3.2f", meanNumMU);
		fprintf (filep, "\n Exp number of mutation events [N=con, ISM]   =   %3.2f", expNumMU);
		if (numDataSets > 1)
			{
			fprintf (filep, "\n Variance number of mutation events           =   %3.2f", varNumMU);
			fprintf (filep, "\n Expected variance number of mutation events  =   %3.2f", expVarNumMU);
			fprintf (filep, "\n    [N=con, no rec, ISM]");
			}
		fprintf (filep, "\n Mean number of SNPs                          =   %3.2f", meanNumSNPs);
		if (numDataSets > 1)
			fprintf (filep, "\n Variance number of SNPs                      =   %3.2f", varNumSNPs);
		fprintf (filep, "\n Proportion of JC sites                       =   %3.2f", propJCsites);
		if (propJCsites > 0)
			fprintf (filep, "\n Relative JC/ISM mutation rate                =   %2.1e", JCrelMutationRate);
		fprintf (filep, "\n Proportion of reps without SNP sites         =   %3.2f", zeroSNPs/(double)numDataSets);
		}
}




/***************************** PrintHeader *******************************/
/* Prints program header */

static void PrintHeader(FILE *filep)
{
	FILE *file = filep;
	fprintf (filep,"SNPs simulation  - %s", PROGRAM_NAME);
	fprintf (filep,"  %s", VERSION_NUMBER);
	fprintf (filep,"\n (c) 2002-2003 David Posada - dposada@variagenics.com");
	fprintf (filep,"\n     with much help from Carsten Wiuf");
	fprintf (filep,"\n__________________________________________________\n\n");
}




/***************************** PrintMemry *******************************/
/* Prints available memory */

static void PrintMemory (FILE *filep)
{

	fprintf (filep, "\nDynamic memory available = %.1lf Mbyte\n", MemAvailable(1024)/1.0e6);

}

/***************************** PrintDate *******************************/
/* Prints current date and time  */

static void PrintDate (FILE *filep)
{
	FILE *file = filep;
	time_t now;
	char *date;

	now=time(NULL);
	date= ctime(&now);
	fprintf (filep,"%s",date);
}



/***************************** PrintUsage *******************************/
/* Prints a short description of program usage  */

static void PrintUsage()
{
	fprintf (stderr, "\n\nUsage: %s [-n# -s# -l# -e# -d# (# # #) -b# -r# -k# -q# -h# -v# -m# -t* -u# -x# -p# -w# -y#  -a -f -i -j -o -1 -##  -?]", PROGRAM_NAME);
	fprintf (stderr,"\n-n: number of replicates (e.g. -n1000)");
	fprintf (stderr,"\n-s: sample size (#haplotypes) (e.g. -s8)");
	fprintf (stderr,"\n-l: number of sites (e.g. -l500)");
	fprintf (stderr,"\n-e: effective population size (e.g. -e1000)");
	fprintf (stderr,"\n-d: number of demographic periods followed by effective  population size at the beginning and at the end of the period, ");
	fprintf (stderr, "\n    and the duration of the period in generations. (e.g. -d1 100 200 30000) (e.g. -d2 100 100 40000 1000 2000 20000)");
	fprintf (stderr,"\n-b: growth rate (e.g. -b1e-6)");
/*	fprintf (stderr,"\n-c: carrying capacity for logistic model (e.g. -c2000)"); */
	fprintf (stderr,"\n-r: homogeneous recombination rate (e.g. -r1e-6)");
	fprintf (stderr,"\n-k: hotspot recombination rate (e.g. -k1e-4)");
	fprintf (stderr,"\n-q: fixed number of hotspots (e.g. -q4)");
	fprintf (stderr,"\n-h: expected number of hotspots (e.g. -h4)");
	fprintf (stderr,"\n-v: hotspot imprecision (e.g. -v100)");
	fprintf (stderr,"\n-m: hotspot width (e.g. -m20)");
	fprintf (stderr,"\n-t: hotspot heterogeneity (e.g. -t0.5)");
	fprintf (stderr,"\n-t: hotspot interference (e.g. -z4)");
	fprintf (stderr,"\n-u: mutation rate (e.g. -u1e-8)");
	fprintf (stderr,"\n-x: simulate a fixed number of SNPs (e.g. -x100)");
	fprintf (stderr,"\n-p: proportion of fast JC sites (e.g. -p0.2)");
	fprintf (stderr,"\n-w: relative JC/ISM mutation rate (e.g. -w2.5)");
	fprintf (stderr,"\n-a: print sequences to a file (e.g. -a)");
	fprintf (stderr,"\n-f: print genotypes to a file (e.g. -f)");
	fprintf (stderr,"\n-i: print trees to a file (e.g. -i)");
	fprintf (stderr,"\n-j: print times to a file (e.g. -j)");
	fprintf (stderr,"\n-o: print site mutation model to a file (e.g. -o)");
	fprintf (stderr,"\n-1: do not simulate data (e.g. -1)");
	fprintf (stderr,"\n-g: number of nodes to allocate (e.g. -g5000)");
	fprintf (stderr,"\n-y: noisy (e.g. -y2)");
	fprintf (stderr,"\n      = 0: does not print anything");
	fprintf (stderr,"\n      = 1:  + simulation summary"); 
	fprintf (stderr,"\n      = 2:  + replicate information"); 
	fprintf (stderr,"\n      = 3: + calculation status and event information");
	fprintf (stderr,"\n      = 4: + print ancestral status at each event + MRCA");
	fprintf (stderr,"\n      = 5: + g's and G");
	fprintf (stderr,"\n-#: seed (e.g. -#37864287)");
	fprintf (stderr,"\n-?: Print help");
	
	fprintf (stderr,"\n\nDefaults: ");
	PrintDefaults (stderr);
	fprintf (stderr,"\n\n");

	exit(-1);
}

/***************** PrintDefaults **********************/
/*	Prints the default settings */

static void PrintDefaults (FILE *fp)
{
	int		i;
	
	fprintf (fp, " -n%d -s%d -l%d -e%d -d%d", numDataSets, numGametes, numSites, N, numPeriods);
	for (i=1; i<=numPeriods; i++)
		fprintf (fp, " %d %d %d", Nbegin[i], Nend[i], cumDuration[i]-cumDuration[i-1]);	
	fprintf (fp, " -b%2.1e",growthRate); 
	fprintf (fp, " -c%d", carryingCapacity);
	fprintf (fp, " -r%2.1e -k%2.1e", recombinationRate, blockRecombinationRate);
	fprintf (fp, " -q%2.1e", fixedNumHotspots);
	fprintf (fp, " -h%2.1e", expNumPoissonHotspots);
	fprintf (fp, " -v%2.1e", hotspotImprecision);
	fprintf (fp, " -m%d", hotspotWidth);
	fprintf (fp, " -z%d", interferenceInterval);
	fprintf (fp, " -u%2.1e", mutationRate);
	fprintf (fp, " -t%2.1e",  alpha);
	fprintf (fp, " -x%d", numSNPs);
	fprintf (fp, " -p%2.1e -w%2.1e -y%d -g%d -#[time(NULL)]", propJCsites, JCrelMutationRate, noisy, numNodes);
}




/***************** PrintCommandLine **********************/
/*	Prints the settings used in the simulations */

static void PrintCommandLine (FILE *fp, long int seed)
{
	int		i;
	
	fprintf (fp, " -n%d -s%d -l%d -e%d -d%d", numDataSets, numGametes, numSites, N, numPeriods);
	for (i=1; i<=numPeriods; i++)
		fprintf (fp, " %d %d %d", Nbegin[i], Nend[i], cumDuration[i]-cumDuration[i-1]);	
	fprintf (fp, " -b%2.1e",growthRate); 
	if (doLogistic == YES)
		fprintf (fp, " -c%d", carryingCapacity);
	fprintf (fp, " -r%2.1e -k%2.1e", recombinationRate, blockRecombinationRate);
	if (fixedNumHotspots > 0)
		fprintf (fp, " -q%2.1e", fixedNumHotspots);
	else
		fprintf (fp, " -h%2.1e", expNumPoissonHotspots);
	if (hotspotUniform == NO)
		fprintf (fp, " -v%2.1e", hotspotImprecision);
	else
		fprintf (fp, " -m%d", hotspotWidth);
	if (hotspotInterference == YES)
		fprintf (fp, " -z%d", interferenceInterval);
	fprintf (fp, " -u%2.1e", mutationRate);
	if (hotspotHeterogeneity == YES)
		fprintf (fp, " -t%2.1e",  alpha);
	if (simulateJustSNPs == YES)
		fprintf (fp, " -x%d", numSNPs);
	fprintf (fp, " -p%2.1e -w%2.1e -y%d -g%d -#%d", propJCsites, JCrelMutationRate, noisy, numNodes, seed);

}


/**************** RelabelNodes **************/
/*	After getting rid of superfluos node, we
	need to relabel those so they are consecutive 
	Label tips with indexes to make sure that in
	case of recombination adjacent trees differ
	at most in one subtree
*/

void RelabelNodes (TreeNode *p)
{
	if (p != NULL)
		{
		RelabelNodes (p->left);
		RelabelNodes (p->right);
		if (p->left == NULL && p->right == NULL) /* is tip */
			{
			/* p->label = tipLabel++; */
			p->label = p->index;
			}
		else
			{
			p->label = intLabel++;
			}
		}
}



/***************** Index ***************/
/* Returns index for a given node */

int Index (TreeNode *p)
{
	return (p == NULL) ? -1 : p->index+1;
}



/***************** Label ***************/
/* Returns label for a given node */

int Label (TreeNode *p)
{
	return (p->anc == NULL && p->left == NULL && p->right == NULL) ? -1 : p->label+1;
}



/*********************** RandomizeArray ***********************/
/* Randomizes elements of a given array */

static void RandomizeArray (int array[],int first, int last, long int *seed)
{
	int i, r, tmp, length;
	
	length = last - first;
	for (i=first; i<last; i++)          /* iterate through the array */
		{               
		r = RandomUniformTo(length-(i-first), seed) + i; /* get random number */
		tmp  = array[r];                /* swap elements */
		array[r] = array[i];
		array[i] = tmp ;
		}
}




/***************************** RandomUniform **********************************/
/*  It returns a random uniform variate in range 0..1. It is described in
  	Park, S. K. and K. W. Miller.  1988.  Random number generators: good
    ones are hard to find.  Communications of the ACM, 31(10):1192-1201.
*/

double RandomUniform (long int *seed)
{

	long int	lo, hi, test;
	
	hi = (*seed) / 127773;
	lo = (*seed) % 127773;
	test = 16807 * lo - 2836 * hi;
	if (test > 0)
		*seed = test;
	else
		*seed = test + 2147483647;
	return (double)(*seed) / (double)2147483647;
}



/********************** RandomUniformTo ****************************/
/* it returns random uniform in range 0...max-1          */

static int RandomUniformTo (int max, long int *seed)
{
	double	rd;
	rd = RandomUniform (seed);
	return (floor(rd*max));
}




/********************* RandomPoisson ********************/
/* Generates a random number from a Poisson distibution with
   mean lambda. 
*/

int RandomPoisson (double lambda, long int *seed)
{
	int		poissonNumber;
	double	sum;
	
	sum = 0;
	poissonNumber = -1;
	
	while (sum <= 1.0)
		{
		sum += RandomExponential (lambda, seed);
		poissonNumber++;
		}

	return poissonNumber;

}


/********************* RandomExponential ********************/
/* Generates a random number from a Poisson distibution with
   mean lambda. 
*/

double RandomExponential (double lambda, long int *seed)
{

	double 	exponentialNumber, U;

	do
		U = RandomUniform (seed);
	while (U == 0);

	exponentialNumber = -log (U) / lambda;

	return exponentialNumber;
}






/********************* RandomNormal ********************/
/*	Generates a normal number using the Box-Mueller scheme 
	(Press et al. 1986 , 202)
*/

double RandomNormal (double mean, double variance, long int *seed)
{

	double 	pi = 3.141592653589793238462643;
	double 	normalNumber, U1, U2, Z;

	U1 = RandomUniform(seed);
	U2 = RandomUniform(seed);
	
	Z = sqrt (-2 * log (U1)) * cos (2*pi*U2);

	normalNumber = mean + sqrt(variance)*Z;

	return normalNumber;
}



/**************************** RandomGamma *************************/
/*	Generates a gamma number using routines in Ziheng's
	Yang tools.h in PAML

	Random standard gamma (Mean=Var=s,  with shape par=s, scale par=1)
	r^(s-1)*exp(-r)
	
	J. Dagpunar (1988) Principles of random variate generation,
	Clarendon Press, Oxford
	
	Calling rndgamma1() if s<1 or rndgamma2() if s>1 or exponential if s=1
*/	

double	RandomGamma (double shape, long int *seed)
{
	double gammaNumber = 0;
	
	if (shape <= 0)
		fprintf (stderr, "ERROR: problems with gamma variable generation, shape < 0");
	else if (shape < 1)
		gammaNumber = RandomGamma1 (shape, seed);
	else if (shape > 1)
		gammaNumber = RandomGamma2 (shape, seed);
	else
		gammaNumber = -log (RandomUniform(seed));
	
	return (gammaNumber);


}

/*************** RandomGamma1 ***************/
double RandomGamma1 (double s, long int *seed)
{
/* Random standard gamma for s<1
   switching method
*/
	double			r, x=0.0, small=1e-37, w;
	static double   a, p, uf, ss=10.0, d;
	
	if (s!=ss) 
		{
		a  = 1.0-s;
		p  = a/(a+s*exp(-a));
		uf = p*pow(small/a,s);
		d  = a*log(a);
		ss = s;
		}
	for (;;) 
		{
		r = RandomUniform(seed);
		if (r > p)        
			x = a-log((1.0-r)/(1.0-p)), w=a*log(x)-d;
		else if (r>uf)  
			x = a*pow(r/p,1/s), w=x;
		else            
			return (0.0);
		r = RandomUniform(seed);
		if (1.0-r <= w && r > 0.0)
		if (r*(w+1.0) >= 1.0 || -log(r) <= w)  
			continue;
		break;
		}
	return (x);
}


/*************** RandomGamma2 ***************/
double RandomGamma2 (double s, long int *seed)
{
/* Random standard gamma for s>1
   Best's (1978) t distribution method
*/
	double			r ,d, f, g, x;
	static double	b, h, ss=0;
	
	if (s!=ss) 
		{
		b  = s-1.0;
		h  = sqrt(3.0*s-0.75);
		ss = s;
		}
	for (;;) 
		{
		r = RandomUniform(seed);
		g = r-r*r;
		f = (r-0.5)*h/sqrt(g);
		x = b+f;
		if (x <= 0.0) 
			continue;
		r = RandomUniform(seed);
		d = 64*r*r*g*g*g;
		if (d*x < x-2.0*f*f || log(d) < 2*(b*log(x/b)-f))  
			break;
		}
	return (x);
}




/**************************** TryToAlloc ********************************/
/* Tries to allocate a certain amount of dynamic memory (Borrowed from D.Clayton) */

int TryToAlloc (long request)
{
	char *c;

	c = malloc (request);

	if (c == NULL)
		return NO;
	else
		{
		free(c);
		return YES;
		}
}


/**************************** MemAvailable ********************************/
/* Determine how much dynamic memory is available (Borrowed from D.Clayton) */

long MemAvailable (int atom) 
{
	long step, res;

	res = 0;
	step = atom;

	while (TryToAlloc (res+step) == YES)
		{
		res += step;
		step *= 2;
		}

	while (step > atom)
		{
		step /= 2;
		if (TryToAlloc (res+step) == YES) 
		  res += step;
		}

	return res;
}



/************************ ReadUntil **************************/
/* Utility to skip comments between brackets. Borrowed from Andy
Rambaut and Nick Grassly */

void ReadUntil(FILE *fv, char stopChar, char *what)
{
	char ch;
	
	ch=fgetc(fv);
	while (!feof(fv) && ch!=stopChar) 
		ch=fgetc(fv);

	if (feof(fv) || ch!=stopChar) {
		fprintf (stderr, "%s missing", what);
		exit(0);
	}
}

/********************************* Sioux *****************************/
/* Macintosh Sioux console settings */

#ifdef macintosh 
static void Sioux()
{
	/*
	Dont exit the program after it runs or ask whether to save the
	window when the program exit */
	SIOUXSettings.autocloseonquit = NO;
	SIOUXSettings.asktosaveonclose = NO;

	/* Dont show the status line */
	SIOUXSettings.showstatusline = YES;

	/* Make the window large enough to fit 1 line of text that
	contains 12 characters. */
	SIOUXSettings.columns = 80;
	SIOUXSettings.rows = 60;

	/* Place the windows top left corner at (5,40). */
	SIOUXSettings.toppixel = 100;
	SIOUXSettings.leftpixel = 450;

	SIOUXSetTitle("\pSNPsim");

	/* Set the font to be 10-point, bold, italic Times. */
	/*SIOUXSettings.fontsize = 10;*/
	/*SIOUXSettings.fontface = italic + bolf=d;*/
	/*SIOUXSettings.fontid = kFontIDTimes;*/
}
#endif
